/****************************************************************************/
/**
* \file rtc_control.h
*
* \brief Contains the main definitions for the real time clock module.
*
* \author   Michael Skutek
* \date	    20.03.2013
* \version  0.3
*
* Platform:    AVR ATmega (tested on ATmega168)
*
* Include Files:
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program. If not, see
* <http://www.gnu.org/licenses/>.
*
****************************************************************************/


#ifndef RTC_CONTROL_H_INCLUDED
#define RTC_CONTROL_H_INCLUDED



/*********************************************************************************************/
/**
*
* \mainpage RTC Module
* <center>$Revision: 0.3 $ </center>
*
* This documentation describes the source code for the Real Time Clock Module with an I2C
* interface. This source code was written and tested for an ATmega168 controller. \n
* The RTC module can be accessed via I2C, at the address defined by I2C_SLAVE_ADDRESS in i2c.h.
* The following registers are available for reading and/or writing:
* \verbatim
 Address         Bit                Value                               Range           Access

  0x00           0...7             current register address           [see i2c.h]         R/W

  0x01            0                current_dcf_signal                   [0, 1]             R
                  1                dcf_enabled                          [0, 1]            R/W
                 2...6             dcf_state                          [see dcf77.h]        R
                  7                set_time_command                     [0, 1]             W

  0x02           0...5             seconds                             [0...59]           R/W
                 6...7             minutes (0...1)                     [0...59]           R/W

  0x03           0...3             minutes (2...5)                     [0...59]           R/W
                 4...7             month                               [1...12]           R/W

  0x04           0...4             hours                               [0...24]           R/W
                 5...7             day of week                        [see dcf77.h]       R/W

  0x05           0...4             day                                 [0...31]           R/W

  0x06           0...6             year                                [0...99]           R/W
  \endverbatim
*
* For example, a master can read the time information (using the provided I2C interface
* function i2c_master_io in i2c.c) from this module in this way:
*
\code
    #define I2C_RTC_SLAVE_ADDRESS 0xaa //Slave Address for RTC on I2C Bus (including Write Bit)
                                       //(7 Bit address + Write Bit)

    uint8_t rtc_data[I2C_MASTER_BUFFER_LENGTH];
    uint8_t i2c_retry_counter;
    real_time_data real_time;          //see dcf77.h
    uint8_t return_value;


    ///get current time and date:
    /// 1. set register address
    /// 2. read data
    /// 3. extract time/date information


    //set address///////////////////////////////////////////////////

    rtc_data[0] = I2C_RTC_SLAVE_ADDRESS; //SLA+W
    rtc_data[1] = 0x00;

    //I2C communication
    i2c_retry_counter = I2C_MAX_RETRYS;
    return_value = I2C_IO_ERROR;

    while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
      return_value = i2c_master_data_io(rtc_data, 2, FALSE);
      i2c_retry_counter--;
      _delay_us(I2C_RETRY_WAIT_US);
    }


    //read from data registers if setting address was successful////

    if (return_value == I2C_IO_SUCC) {

      rtc_data[0] = I2C_RTC_SLAVE_ADDRESS | 0x01; //SLA+R

      //I2C communication
      i2c_retry_counter = I2C_MAX_RETRYS;
      return_value = I2C_IO_ERROR;

      while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
        return_value = i2c_master_data_io(rtc_data, 7, FALSE);
        i2c_retry_counter--;
        _delay_us(I2C_RETRY_WAIT_US);
      }

    }


    //extract time/date information/////////////////////////////////

    if (return_value == I2C_IO_SUCC) {

      //extract real time from I2C registers
      real_time.current_second = rtc_data[2] & 0x3f;
      real_time.current_minute = (rtc_data[2] >> 6) & 0x03;
      real_time.current_minute = real_time.current_minute | ((rtc_data[3] & 0x0f) << 2);
      real_time.current_month = (rtc_data[3] >> 4) & 0x0f;
      real_time.current_hour = rtc_data[4] & 0x1f;
      real_time.current_weekday = (rtc_data[4] >> 5) & 0x07;
      real_time.current_day = rtc_data[5] & 0x1f;
      real_time.current_year = rtc_data[6] & 0x7f;

    }
\endcode
* If enabled, the current time and date is also printed on a LCD (currently 8x2 characters).
* Therefore, the definition RTC_LCD_TIMEDEBUG should be set to "TRUE". See also the
* file "port_definitions.h" for the pin/port settings. \n\n
* If the definition of RTC_SAA1064_OUTPUT is set to "1", the module sends the time data
* to three SAA1064 chips for displaying the time via a 7 segment LED displays (each SAA1064
* controls two 7 segment LED displays separately for hours, minutes or seconds). The interface
* to the SAA1064 chips is the same I2C bus, the I2C adresses of these chips are defined in
* the file saa1064.h (I2C_SAA1064_ADDRESS1 for hours, I2C_SAA1064_ADDRESS2 for minutes and
* I2C_SAA1064_ADDRESS3 for seconds). \n\n
* For a detailed description of the functions, see the function description, starting with
* main()... \n
*
*/
/*********************************************************************************************/




/******************************************
	\internal
	General Definitions
*******************************************/

#ifndef TRUE
#define TRUE                             1          //!< value for TRUE
#endif
#ifndef FALSE
#define FALSE                            0          //!< value for FALSE
#endif


#define RTC_CONTROL_VERSION            0.3          //!< software version of RTC module


#define RTC_LCD_DEBUG                    0          //!< enables or disables debugging with LCD output [0, 1]
#define RTC_LCD_TIMEDEBUG                0          //!< enables or disables time data output with LCD output [0, 1]

#define RTC_OCR0A                      195          //!< value for compare match (156 = 10 ms for 16 MHz clock, 195 = 10 ms for 20 MHz clock)

#define RTC_BASE_YEAR                 2000          //!< base year for realtime


#define RTC_SAA1064_OUTPUT               1          //!< time data output on 7 segment LED via SAA1064


#define MAX_TIME_SCALE                 250          //!< maximum time scale [10 ms]

#define DCF77_SAMPLE_TIME                1          //!< sample time for DCF77 signal [10 ms]

#define DCF77_UPDATE_CYCLE              30          //!< update cycle for DFC sync [min]
#define DCF77_ENABLED_DEFAULT            1          //!< default value for DCF enabled (1) or not (0)

#define DCF77_CYCLE_SECONDS             58          //!< current second after complete received cycle


#define LED_HIGH_CURRENT_START_TIME  72000UL        //!< start time for high LED current (==day time) - ATTENTION: no leading zero (0)!
#define LED_LOW_CURRENT_START_TIME  214500UL        //!< start time for low LED current (==night time) - ATTENTION: no leading zero (0)!



/******************************************
	\internal
	Definitions for I2C interface to master
*******************************************/

//requests from base
#define RTC_CTRL_GETTIME              0x01          //!< get time command
#define RTC_CTRL_CONFIG               0x02          //!< set config command


#define I2C_SLAVE_TRANSFER_ABORT_CYCLES    30       //!< time (timer counts) to wait until slave receive/transmission will be aborted
//only used, if master I/O has higher priority!


/******************************************
	\internal
	Structure definitions
*******************************************/

typedef struct dcf77_config_struc {

  uint8_t current_dcf_signal;                       //!< contains the current DCF77 signal value [0, 1]
  uint8_t dcf_state;                                //!< contains the return value (internal state) of the function dcf77_sync()
                                                    //!  (see the internal state definitions in file dcf77.h)
  uint8_t dcf_enabled;                              //!< enables or disables the DCF77 data aquisition [DCF77_ENABLED_DEFAULT]

} dcf77_config_data;


/******************************************
	\internal
	Function prototypes
*******************************************/




/******************************************
	\internal
	Register description
*******************************************/

// Address         Bit                Value                               Range           Access
//
//   0             0...7             current register address           [see i2c.h]         R/W
//
//   1              0                current_dcf_signal                   [0, 1]             R
//                  1                dcf_enabled                          [0, 1]            R/W
//                 2...6             dcf_state                          [see dcf77.h]        R
//                  7                set_time_command                     [0, 1]             W
//
//   2             0...5             seconds                             [0...59]           R/W
//                 6...7             minutes (0...1)                     [0...59]           R/W
//
//   3             0...3             minutes (2...5)                     [0...59]           R/W
//                 4...7             month                               [1...12]           R/W
//
//   4             0...4             hours                               [0...24]           R/W
//                 5...7             day of week                        [see dcf77.h]       R/W
//
//   5             0...4             day                                 [0...31]           R/W
//
//   6             0...6             year                                [0...99]           R/W





#endif // RTC_CONTROL_H_INCLUDED
