/****************************************************************************/
/**
* \file saa1064.h
*
* \brief routines for LED control (via SAA1064)
*
* \author   Michael Skutek
* \date	    02/23/2013
* \version  0.2
*
* Platform:    AVR ATmega (tested on ATmega168)
*
* Include Files:
*       - avr/pgmspace.h:       for accessing program space
*       - util/delay.h:         time delays (part of avr-gcc)
*       - i2c.h                 I2C routines
*       - saa1064.h:            definitions and prototypes
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program. If not, see
* <http://www.gnu.org/licenses/>.
*
****************************************************************************/


#ifndef SAA1064_H_INCLUDED
#define SAA1064_H_INCLUDED


/******************************************
	\internal
	Internal value definitions
*******************************************/


///definitions for SAA1064 I/O

//LED current
#define SAA1064_LED_CURRENT01           0x01          //LED current (0 0 1 == 0 * 12mA 0 * 6mA 1 * 3mA)
#define SAA1064_LED_CURRENT02           0x02          //LED current (0 1 0 == 0 * 12mA 1 * 6mA 0 * 3mA)
#define SAA1064_LED_CURRENT03           0x03          //LED current (0 1 1 == 0 * 12mA 1 * 6mA 1 * 3mA)
#define SAA1064_LED_CURRENT04           0x04          //LED current (1 0 0 == 0 * 12mA 0 * 6mA 0 * 3mA)
#define SAA1064_LED_CURRENT05           0x05          //LED current (1 0 1 == 1 * 12mA 0 * 6mA 1 * 3mA)
#define SAA1064_LED_CURRENT06           0x06          //LED current (1 1 0 == 1 * 12mA 1 * 6mA 0 * 3mA)
#define SAA1064_LED_CURRENT07           0x07          //LED current (1 1 1 == 1 * 12mA 1 * 6mA 1 * 3mA)

//return values
///IMPORTANT: THESE RETURN VALUES MUST DIFFER FROM I2C RETURN VALUES (see "i2c.h")
#define SAA1064_IO_SUCCESS       0xe0     //i/o operation was successful
#define SAA1064_WRONG_ADDRESS    0xe1     //wrong I2C adress
#define SAA1064_WRONG_MODE       0xe2     //wrong value for display mode
#define SAA1064_INIT_ERROR       0xe3     //error during initialization of SAA1064 (invalid control byte data received)
#define SAA1064_IO_ERROR         0xe4     //error during SAA1064 I/O (internal/unknown error - e.g. wrong register length)


#define SAA1064_NUMBER_RETRIES      3     //number of retries if I2C Bus is blocked (see also I2C_RETRY_WAIT_US in "i2c.h")

#define SAA1064_INIT_DELAY_MS   500.0     //delay during two initialization messages [ms]
#define SAA1064_DEMO_DELAY_MS    50.0     //delay during two demo messages [ms]
#define SAA1064_DEMO_LOOPS          5     //number of demo loops


///Do not change these values!
#define SAA1064_STATIC_MODE      0x00     //static mode of SAA1064 (C0 = 0)
#define SAA1064_DYNAMIC_MODE     0x01     //dynamic mode of SAA1064 (C0 = 1)

//address values
///Do not change the address values!
#define I2C_SAA1064_ADDRESS0      0x70     //I2C address (including write bit "0") for ADR Pin = 0 V
#define I2C_SAA1064_ADDRESS1      0x72     //I2C address (including write bit "0") for ADR Pin = 3/8 Vcc
#define I2C_SAA1064_ADDRESS2      0x74     //I2C address (including write bit "0") for ADR Pin = 5/8 Vcc
#define I2C_SAA1064_ADDRESS3      0x76     //I2C address (including write bit "0") for ADR Pin = Vcc

#define I2C_SAA1064_NUMBER_REG       7     //number of data registers including address
#define I2C_SAA1064_NUMBER_STAT      2     //number of status registers including address




/// 7-Segment Display Configuration
///
/// Display Type: Numitron IV-9
///
/// Code:  Segment ID/(Port ID)
///
///
///
///               5/(1)
///       ______________________
///      |                      |
///      |                      |
///      |                      |
///      |                      |
///      |                      |
///      | 6/(6)                | 3/(2)
///      |                      |
///      |                      |
///      |                      |
///      |        7/(7)         |
///      |______________________|
///      |                      |
///      |                      |
///      |                      |
///      |                      |
///      |                      |
///      | 9/(5)                | 4/(3)
///      |                      |
///      |                      |
///      |                      |
///      |         8/(4)        |      __
///      |______________________|     |__|  2/(8)


/// Segments (Port Value)
///
/// 0: 3, 4, 5, 6, 8, 9         (00111111)
/// 1: 3, 4                     (00000110)
/// 2: 3, 5, 7, 8, 9            (01011011)
/// 3: 3, 4, 5, 7, 8            (01001111)
/// 4: 3, 4, 6, 7               (01100110)
/// 5: 4, 5, 6, 7, 8            (01101101)
/// 6: 4, 5, 6, 7, 8            (01111101)
/// 7: 3, 4, 5                  (00000111)
/// 8: 3, 4, 5, 6, 7, 8, 9      (01111111)
/// 9: 3, 4, 5, 6, 7, 8         (01101111)
/// 0.: 2, 3, 4, 5, 6, 8, 9     (10111111)
/// 1.: 2, 3, 4                 (10000110)
/// 2.: 2, 3, 5, 7, 8, 9        (11011011)
/// 3.: 2, 3, 4, 5, 7, 8        (11001111)
/// 4.: 2, 3, 4, 6, 7           (11100110)
/// 5.: 2, 4, 5, 6, 7, 8        (11101101)
/// 6.: 2, 4, 5, 6, 7, 8        (11111101)
/// 7.: 2, 3, 4, 5              (10000111)
/// 8.: 2, 3, 4, 5, 6, 7, 8, 9  (11111111)
/// 9.: 2, 3, 4, 5, 6, 7, 8     (11101111)
/// .: 2                        (10000000)
/// -: 7                        (01000000)
///  :                          (00000000)



#define BCD_TABLE_LENGTH          23          //number of entries in numbers_bcd_table
#define BCD_TABLE_DECIMAL_POINT_VALUE   20    //entry which contains the decimal point "."
#define BCD_TABLE_ERROR_VALUE     21          //entry which contains the error sign "-"
#define BCD_TABLE_OFF_VALUE       22          //entry which contains no output
#define BCD_DECIMAL_POINT_OFFSET  10          //offset value for all values with decimal point

static const unsigned char numbers_bcd_table[] PROGMEM = {
   0x3f,    //0
   0x06,    //1
   0x5b,    //2
   0x4f,    //3
   0x66,    //4
   0x6d,    //5
   0x7d,    //6
   0x07,    //7
   0x7f,    //8
   0x6f,    //9
   0xbf,    //0.
   0x86,    //1.
   0xdb,    //2.
   0xcf,    //3.
   0xe6,    //4.
   0xed,    //5.
   0xfd,    //6.
   0x87,    //7.
   0xff,    //8.
   0xef,    //9.
   0x80,    //.
   0x40,    //-      (displayed in case of an error)
   0x00     //       (display off)
   };



/******************************************
	\internal
	Function prototypes
*******************************************/

uint8_t saa1064_init(uint8_t, uint8_t, uint8_t);
uint8_t saa1064_read_status(uint8_t, uint8_t *);
uint8_t saa1064_send_digits(uint8_t, uint8_t, uint8_t, uint16_t, uint8_t, uint8_t);



#endif // SAA1064_H_INCLUDED
