/****************************************************************************/
/**
* \file i2c.c
*
* \brief Contains the functions for the data transmission via
*        the I2C interface as master and also as slave.
*
* \author   Michael Skutek
* \date	    20.03.2013
* \version  0.3
*
* Platform:    AVR ATmega (tested on ATmega168)
*
* Include Files:
*       - avr/io.h:             port accessing (part of avr-gcc)
*       - avr/interrupt.h:      interrupt related (part of avr-gcc)
*       - util/delay.h:         time delays (part of avr-gcc)
*       - port_definitions.h:   definition of output ports
*       - rtc_control.h:        all global definitions
*       - i2c.h:                I2C related functions
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program. If not, see
* <http://www.gnu.org/licenses/>.
*
****************************************************************************/


/******************************************
	\internal
	Include files
*******************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "basic_definitions.h"
#include "port_definitions.h"
#include "i2c.h"

#if RTC_LCD_DEBUG == 1
#include <stdlib.h>
#include "lcd.h"
#endif


/******************************************
	\internal
	Global variables
*******************************************/

extern volatile uint8_t i2c_slave_mode_blocked;         // device is blocked an can not receive data from I2C master




/****************************************************************************/
/**
* Initializes the I/O pins SCL and SDA for the I2C interface. This function
* must be called BEFORE any I2C interface function can be used. \n
*
*
* \date
* - Created 16.05.2010
* - Modified  016.05.2010
*
* \see port_definitions.h
*
* \author
*   Michael Skutek, michael@skutek.de
*/
/****************************************************************************/

void i2c_pin_init(void) {

  //pin initialization for I2C bus
  I2C_PORT = I2C_PORT & ~(1 << I2C_SDA_ADDR) & ~(1 << I2C_SCL_ADDR);

}


/****************************************************************************/
/**
* Sets the SCL frequency (registers according to requested frequency in
* master mode). This function must be called BEFORE any I2C interface
* function can be used in master mode. For slave mode, the SCL frequency is
* set automatically.         \n
*
*
* \param [in] f_cpu         CPU frequency [Hz]
* \param [in] f_scl         SCL frequency [Hz]
*
*
* \date
* - Created 16.05.2010
* - Modified  16.05.2010
*
*
* \author
*   Michael Skutek, michael@skutek.de
*/
/****************************************************************************/

uint8_t i2c_set_scl(uint32_t f_cpu, uint32_t f_scl) {

  uint32_t uint32_tmp;


  // calculate first part of equation
  uint32_tmp = (f_cpu / f_scl) / 2;

  //check for maximum value
  if (uint32_tmp <= 8) {           //value too big - set it to maximum
    TWSR = TWSR & ~(1 << TWPS1) & ~(1 << TWPS0);    //Prescaler value: 1
    TWBR = 0x00;                                    //Bitrate value: 0
    return I2C_SCL_ERR;
  }

  //check for needed prescaler value
  uint32_tmp = uint32_tmp - 8;

  if (uint32_tmp <= I2C_MAX_PRESCALER_1) {          //Prescaler value: 1
    TWSR = TWSR & ~(1 << TWPS1) & ~(1 << TWPS0);    //Prescaler value: 1
    TWBR = uint32_tmp;                              //Bitrate value
  } else if (uint32_tmp <= I2C_MAX_PRESCALER_4) {
    TWSR = TWSR & ~(1 << TWPS1);                    //Prescaler value: 4
    TWSR = TWSR | (1 << TWPS0);
    TWBR = uint32_tmp / 4;                          //Bitrate value
  } else if (uint32_tmp <= I2C_MAX_PRESCALER_16) {
    TWSR = TWSR | (1 << TWPS1);                     //Prescaler value: 16
    TWSR = TWSR & ~(1 << TWPS0);
    TWBR = uint32_tmp / 16;                         //Bitrate value
  } else if (uint32_tmp <= I2C_MAX_PRESCALER_64) {
    TWSR = TWSR  | (1 << TWPS1) | (1 << TWPS0);     //Prescaler value: 64
    TWBR = uint32_tmp / 64;                         //Bitrate value
  } else {
    TWSR = TWSR  | (1 << TWPS1) | (1 << TWPS0);     //Prescaler value: 64
    TWBR = 255;                                     //Bitrate value
    return I2C_SCL_ERR;                            //value too small - set it to minimum
  }

  return I2C_SCL_SET_SUCC;

}


/****************************************************************************/
/**
* Sends or receives data from master to slave or vice versa via I2C.
* This function handles the complete communication with a slave. \n
*
*
* \param [in] data          array with data bytes: \n
*                           [slave_address + Read/Write-Bit, \n
*                           register address, \n
*                           data byte, \n
*                           data byte, \n
*                           data byte ... ] \n
* \param [in] number_bytes  number of data bytes in array to read or write
* \param [in] end_noack     slave sends NOACK after receiving last byte
*                           (TRUE), or a NOACK will be always interpreted
*                           as an error value from the slave (FALSE)
*
*
* \date
* - Created 16.05.2010
* - Modified  16.05.2010
*
* \see i2c.h,I2C_MAX_TIMEOUT_WAIT_CYCLES, I2C_TIMEOUT_RETRY_WAIT_US,
*      I2C_STA_RETRY_WAIT_US, the interrupt service routine for the slave mode
*      I2C communication in the file isr.c
*
* \return I2C return codes for master mode (see the respective part in i2c.h)
*
* \author
*   Michael Skutek, michael@skutek.de
*/
/****************************************************************************/

uint8_t i2c_master_data_io(uint8_t *data, uint8_t number_bytes, uint8_t end_noack) {

  uint8_t i;
  uint8_t status;
  static uint8_t TWIE_tmp;
  static uint8_t TWEA_tmp;
  static uint8_t SREG_tmp;
  uint8_t i2c_retry_counter;

#if RTC_LCD_DEBUG == 1
  static uint16_t i2c_error_counter = 0;
  static char tmp_char[40];
#endif


  /******************************************
  \internal
  check for correct input data
  *******************************************/

  if ((number_bytes < 2) || (number_bytes > I2C_MASTER_BUFFER_LENGTH)) {
    return I2C_IO_NUM_ERROR;
  }

  /******************************************
  \internal
  switch off global interrupts
  *******************************************/

  SREG_tmp = SREG;    //save SREG
  cli();              //clear global interrupt enable


  /******************************************
  \internal
  check for pending slave mode communication
  *******************************************/

  if (i2c_slave_mode_blocked == 1) {
    SREG = SREG_tmp;
    return I2C_IO_BLOCK_ERROR;
  } else {
    TWIE_tmp = (TWCR >> TWIE) & 0x1;      //save TWIE bit
    TWCR = TWCR & ~(1 << TWIE);
    TWEA_tmp = (TWCR >> TWEA) & 0x1;      //save TWEA bit
    TWCR = TWCR & ~(1 << TWEA);
    SREG = SREG_tmp;
  }


#if RTC_LCD_DEBUG == 1
  if (TWEA_tmp != 1) {
    lcd_set_cursor(16, 2);
    lcd_send_string("!A", FALSE);
    itoa(i2c_error_counter, tmp_char, 10);
    lcd_send_string(tmp_char, FALSE);
  }
#endif

  //wait for a short time
  _delay_us(I2C_WAIT_BEFORE_STA_US);


  /******************************************
  \internal
  check for free TWI module (no pending
  communication from slave mode...)
  *******************************************/

  if (((TWCR & (1 << TWSTO)) > 0) || ((TWSR & 0xF8) == I2C_BUS_ERROR)) {

#if I2C_RESET_BUS_ON_ERROR == 1
    if ((TWCR & (1 << TWSTO)) > 0) {
      TWCR = 0;
    } else {
      TWCR = (1 << TWSTO);     //send stop condition
    }
#endif
	//clear the TWINT flag
    if ((TWCR & (1 << TWINT)) > 0) {
      TWCR = TWCR | (1 << TWINT);
    }
    //while (!(TWCR & (1 << TWSTO)));
    TWCR = TWCR | (1 << TWEN) | (TWIE_tmp << TWIE) | (TWEA_tmp << TWEA);
#if I2C_RESET_BUS_ON_ERROR != 1
    return I2C_IO_BUS_ERROR;
#endif

  }


  /******************************************
  \internal
  TWINT flag is set --> may be an access from
  another master
  *******************************************/

  if ((TWCR & (1 << TWINT)) > 0) {

    //if no interrupt is enabled --> reset TWINT-Flag - otherwise ISR will do
    if (TWIE_tmp == 0) {
      TWCR = TWCR | (1 << TWINT);
    }
    TWCR = TWCR | (1 << TWEN) | (TWIE_tmp << TWIE) | (TWEA_tmp << TWEA);
    return I2C_IO_BLOCK_ERROR;
  }


  /******************************************
  \internal
  send a start condition and wait for ACK
  *******************************************/

  TWCR = (1 << TWINT) | (1 << TWSTA) | (1 << TWEN);     //start
  i2c_retry_counter = I2C_MAX_STA_WAIT_CYCLES;
  while (!(TWCR & (1 << TWINT)) && (i2c_retry_counter > 0)) {
    i2c_retry_counter--;
    _delay_us(I2C_STA_RETRY_WAIT_US);
  }

  //check for locked bus - reset it if locked
  if (i2c_retry_counter == 0) {
    TWCR = 0;
#if RTC_LCD_DEBUG == 1
    i2c_error_counter++;
    lcd_set_cursor(14, 2);
    lcd_send_string("!S", FALSE);
    itoa(i2c_error_counter, tmp_char, 10);
    lcd_send_string(tmp_char, FALSE);
#endif
    _delay_us(I2C_STA_RETRY_WAIT_US);
    TWCR = TWCR | (1 << TWSTO) | (1 << TWEN) | (TWIE_tmp << TWIE) | (TWEA_tmp << TWEA);
    return I2C_STA_ERROR;
  }

  //check for status
  status = TWSR & 0xF8;
  if ((status != I2C_START_COND_TRANS) && (status != I2C_RSTART_COND_TRANS)) {                //start condition transmitted?

    if (status == I2C_IO_BUS_ERROR) {
      TWCR = (1 << TWINT) | (1 << TWEN) | (1 << TWSTO);   //send stop condition
      //while (!(TWCR & (1 << TWSTO)));
    }

    TWCR = TWCR | (TWIE_tmp << TWIE) | (TWEA_tmp << TWEA);        //restore interrupt flag

    return status;
  };

  /******************************************
  \internal
  send SLA+W or SLA+R and wait for ACK
  *******************************************/

  TWDR = data[0];
  TWCR = (1 << TWINT) | (1 << TWEN);            //send data

  i2c_retry_counter = I2C_MAX_TIMEOUT_WAIT_CYCLES;
  while (!(TWCR & (1 << TWINT)) && (i2c_retry_counter > 0)) {
    i2c_retry_counter--;
    _delay_us(I2C_TIMEOUT_RETRY_WAIT_US);
  }

  //check for locked bus - reset it if locked
  if (i2c_retry_counter == 0) {
    TWCR = TWCR | (1 << TWSTO);     //send stop condition
    _delay_us(I2C_TIMEOUT_RETRY_WAIT_US);
    TWCR = TWCR | (1 << TWEN) | (TWIE_tmp << TWIE) | (TWEA_tmp << TWEA);
    return I2C_TIMEOUT_ERROR;
  }

  //check for status
  status = TWSR & 0xF8;
  if ((status != I2C_SLAW_TRANS) && (status != I2C_SLAR_TRANS)) {             //ACK after slave address transmission received?
    if (status == I2C_M_ARB_LOST) {               //arbitration lost? do nothing (maybe switch to slave mode in calling function)
      TWCR = (1 << TWINT) | (TWIE_tmp << TWIE) | (TWEA_tmp << TWEA);
      return status;
    } else {                                      //send stop condition
      TWCR = (1 << TWINT) | (1 << TWEN) | (1 << TWSTO);
      //while (!(TWCR & (1 << TWSTO)));
      TWCR = TWCR | (TWIE_tmp << TWIE) | (TWEA_tmp << TWEA);
      return status;
    }
  }


  /******************************************
  \internal
  send data to slave
  *******************************************/

  if (status == I2C_SLAW_TRANS) {

    //send all bytes to slave (without SLA+W)
    for (i = 1; i < number_bytes; i++) {

      TWDR = data[i];
      TWCR = (1 << TWINT) | (1 << TWEN);            //send data

      i2c_retry_counter = I2C_MAX_TIMEOUT_WAIT_CYCLES;
      while (!(TWCR & (1 << TWINT)) && (i2c_retry_counter > 0)) {
        i2c_retry_counter--;
        _delay_us(I2C_TIMEOUT_RETRY_WAIT_US);
      }

      //check for locked bus - reset it if locked
      if (i2c_retry_counter == 0) {
        TWCR = TWCR | (1 << TWSTO);     //send stop condition
        _delay_us(I2C_TIMEOUT_RETRY_WAIT_US);
        TWCR = TWCR | (1 << TWEN) | (TWIE_tmp << TWIE) | (TWEA_tmp << TWEA);
        return I2C_TIMEOUT_ERROR;
      }

      status = TWSR & 0xF8;

      if (status != I2C_DATA_TRANS) {              //ACK after data transmission received?
        if (status == I2C_M_ARB_LOST) {            //arbitration lost? do nothing (maybe switch to slave mode in calling function)
          TWCR = (1 << TWINT) | (TWIE_tmp << TWIE) | (TWEA_tmp << TWEA);
          return status;
        } else {                                      //send stop condition
          TWCR = (1 << TWINT) | (1 << TWEN) | (1 << TWSTO);
          //while (!(TWCR & (1 << TWSTO)));
          TWCR = TWCR | (TWIE_tmp << TWIE) | (TWEA_tmp << TWEA);
          if ((i == (number_bytes - 1)) && (end_noack == TRUE)) {              //NOACK from slave after last byte?
            return I2C_IO_SUCC;
          } else {
            return status;
          }
        }
      }

    }

  /******************************************
  \internal
  read data from slave
  *******************************************/

  } else if (status == I2C_SLAR_TRANS) {

    //read only n-1 bytes from slave (without SLA+R
    for (i = 1; i < (number_bytes - 1); i++) {

      TWCR = (1 << TWINT) | (1 << TWEN) | (1 << TWEA);         //send ACK

      i2c_retry_counter = I2C_MAX_TIMEOUT_WAIT_CYCLES;
      while (!(TWCR & (1 << TWINT)) && (i2c_retry_counter > 0)) {
        i2c_retry_counter--;
        _delay_us(I2C_TIMEOUT_RETRY_WAIT_US);
      }

      //check for locked bus - reset it if locked
      if (i2c_retry_counter == 0) {
        TWCR = TWCR | (1 << TWSTO);     //send stop condition
        _delay_us(I2C_TIMEOUT_RETRY_WAIT_US);
        TWCR = TWCR | (1 << TWEN) | (TWIE_tmp << TWIE) | (TWEA_tmp << TWEA);
        return I2C_TIMEOUT_ERROR;
      }

      status = TWSR & 0xF8;

      if (status != I2C_DATA_RCV_ACK) {             //Data received and ACK sent?
        if (status == I2C_M_ARB_LOST) {               //arbitration lost? do nothing (maybe switch to slave mode in calling function)
          TWCR = (1 << TWINT) | (TWIE_tmp << TWIE) | (TWEA_tmp << TWEA);
          return status;
        } else {                                      //send stop condition
          TWCR = (1 << TWINT) | (1 << TWEN) | (1 << TWSTO);
          //while (!(TWCR & (1 << TWSTO)));
          TWCR = TWCR | (TWIE_tmp << TWIE) | (TWEA_tmp << TWEA);
          return status;
        }
      }
      data[i] = TWDR;                               //read data

    }

    //read last byte and send NOACK after it
    TWCR = (1 << TWINT) | (1 << TWEN);              //send NOACK

    i2c_retry_counter = I2C_MAX_TIMEOUT_WAIT_CYCLES;
    while (!(TWCR & (1 << TWINT)) && (i2c_retry_counter > 0)) {
      i2c_retry_counter--;
      _delay_us(I2C_TIMEOUT_RETRY_WAIT_US);
    }

    //check for locked bus - reset it if locked
    if (i2c_retry_counter == 0) {
      TWCR = TWCR | (1 << TWSTO);     //send stop condition
      _delay_us(I2C_TIMEOUT_RETRY_WAIT_US);
      TWCR = TWCR | (1 << TWEN) | (TWIE_tmp << TWIE) | (TWEA_tmp << TWEA);
      return I2C_TIMEOUT_ERROR;
    }

    status = TWSR & 0xF8;

    if (status != I2C_DATA_RCV_NACK) {               //Data received and ACK sent?
      if (status == I2C_M_ARB_LOST) {               //arbitration lost? do nothing (maybe switch to slave mode in calling function)
        TWCR = (1 << TWINT) | (TWIE_tmp << TWIE) | (TWEA_tmp << TWEA);
        return status;
      } else {                                      //send stop condition
        TWCR = (1 << TWINT) | (1 << TWEN) | (1 << TWSTO);
        //while (!(TWCR & (1 << TWSTO)));
        TWCR = TWCR | (TWIE_tmp << TWIE) | (TWEA_tmp << TWEA);
        return status;
      }
    }
    data[i] = TWDR;                                 //read data


  }

  /******************************************
  \internal
  send stop condition (end of transmission)
  *******************************************/
  TWCR = (1 << TWINT) | (1 << TWEN) | (1 << TWSTO);

  //while (!(TWCR & (1 << TWSTO)));

  TWCR = TWCR | (TWIE_tmp << TWIE) | (TWEA_tmp << TWEA);


  return I2C_IO_SUCC;


}


/****************************************************************************/
/**
* Sends or receives data from slave to master or vice versa via I2C.
* This function handles the communication with a master. \n
*
* \note The first byte transmitted from master to the slave sets the
*       register address. After each read or write the pointer
*       to the data registers will be incremented. \n
*       This function can be used in interrupt mode (in cooperation
*       with the respective interrupt service routine in isr.c) or
*       in polling mode with manual checks for a master access and the
*       current state of the communication. \n
*       Depending on the mode (interrupt or polling) the function
*       either will be called by the respective interrup service
*       routine or must be called manually, when the master accesses
*       the slave.
*
*
* \param [in] data          array with data bytes: \n
*                           [slave_address + Read/Write-Bit, \n
*                           register address, \n
*                           data byte, \n
*                           data byte, \n
*                           data byte ... ] \n
* \param [in] number_bytes  number of data bytes in array read or written
*                           from master
* \param [in] command       specifies if slave mode is continued, stopped
*                           after complete transmission or aborted during
*                           the current transmission (see the definitions
*                           for the slave mode commands in i2c.h)
*
*
* \date
* - Created 16.05.2010
* - Modified  16.05.2010
*
* \see i2c.h, I2C_SLAVE_INTERRUPT_OPERATION, the interrupt service routine
*      for the slave mode I2C communication in the file isr.c
*
* \return I2C return codes for slave mode (see the respective part in i2c.h)
*
* \author
*   Michael Skutek, michael@skutek.de
*/
/****************************************************************************/
#if I2C_SLAVE_INTERRUPT_OPERATION == 1
uint8_t i2c_slave_data_io(volatile uint8_t *data, volatile uint8_t *number_bytes, volatile uint8_t command) {
#else
uint8_t i2c_slave_data_io(uint8_t *data, uint8_t *number_bytes, uint8_t command) {
#endif

  static uint8_t address_counter = 0;               //initial value: 0
  uint8_t current_address;
  uint8_t status;
  static uint8_t TWIE_tmp;

#if RTC_LCD_DEBUG == 1
  static uint16_t i2c_error_counter = 0;
  static char tmp_char[40];
#endif



  /******************************************
  \internal
  after being addressed as slave:
  *******************************************/

  //save Interrupt Flag
  TWIE_tmp = (TWCR >> TWIE) & 0x1;

  //check Status Register
  status = TWSR & 0xF8;

  switch (status) {


    /******************************************
    \internal
    SLAVE RECEIVE mode
    *******************************************/


    //SLA + Write Bit received and ACK was returned
  case I2C_SLAW_ACK_RCVD:
    //or Arbitration lost as Master (SLAW), SLAW received and ACK was returned
  case I2C_SLAW_ACK_ARB_LOST:
    //or General Call + Write Bit received and ACK was returned
  case I2C_GCA_ACK_RCVD:
    //Arbitration lost as Master (SLAW), General Call + Write Bit received and ACK was returned
  case I2C_GCA_ACK_ARB_LOST:

    address_counter = 0;                      //set adress counter to 0
    *number_bytes = 0;                        //set number of transferred bytes to 0
    //slave mode aborted or not?
    if (command == I2C_SLAVE_MODE_ABORT) {
      TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE);        //next step: data byte will be received and NOACK returned
    } else {
      TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE) | (1 << TWEA);        //next step: data byte will be received and ACK returned
    }

    break;



    //Data received, ACK returned
  case I2C_DATA_ACK_RCVD:
    //Data received, NOACK returned
  case I2C_DATA_NOACK_RCVD:
    //General Call Data received, ACK returned
  case I2C_GCA_DATA_ACK_RCVD:
    //General Call Data received, NOACK returned
  case I2C_GCA_DATA_NOACK_RCVD:

    //first byte? --> register address
    if (address_counter == 0) {

      data[0] = TWDR;
      *number_bytes = *number_bytes + 1;
      address_counter++;

      //if next byte is last byte of buffer (or overflow)  --> after next byte NOACK will be will be returned
      if ((data[0] + address_counter) > (I2C_SLAVE_BUFFER_LENGTH - 1)) {
        TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE);        //next step: data byte will be received and NOACK returned
      } else {
        //slave mode aborted or not?
        if ((command == I2C_SLAVE_MODE_ABORT) ||
            ((status == I2C_DATA_NOACK_RCVD) && (command == I2C_SLAVE_MODE_END)) ||
            ((status == I2C_GCA_DATA_NOACK_RCVD) && (command == I2C_SLAVE_MODE_END))) {
          TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE);        //next step: data byte will be received and NOACK returned
        } else {
          TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE) | (1 << TWEA);        //next step: data byte will be received and ACK returned
        }
      }

    } else {

      //calculate current address
      current_address = data[0] + address_counter;

      //valid address, also for next byte
      if (current_address < (I2C_SLAVE_BUFFER_LENGTH - 1)) {

        data[current_address] = TWDR;
        *number_bytes = *number_bytes + 1;
        address_counter++;
        //slave mode aborted or not?
        if ((command == I2C_SLAVE_MODE_ABORT) ||
            ((status == I2C_DATA_NOACK_RCVD) && (command == I2C_SLAVE_MODE_END)) ||
            ((status == I2C_GCA_DATA_NOACK_RCVD) && (command == I2C_SLAVE_MODE_END))) {
          TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE);        //next step: data byte will be received and NOACK returned
        } else {
          TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE) | (1 << TWEA);        //next step: data byte will be received and ACK returned
        }

        //current address is valid, but last byte of buffer
      } else if (current_address < I2C_SLAVE_BUFFER_LENGTH) {

        data[current_address] = TWDR;
        *number_bytes = *number_bytes + 1;
        address_counter++;
        TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE);        //next step: data byte will be received and NOACK returned

        //current address is invalid
      } else {

        TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE);        //next step: data byte will be received and NOACK returned

      }

    }

    break;


    /******************************************
    \internal
    END OF TRANSMISSION:
    *******************************************/

    //stop or repeated start condition received (while adressed)
  case I2C_STO_REPSTART_COND:

    //slave mode continued or not?
    if (command == I2C_SLAVE_MODE_CONTINUE) {
      TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE) | (1 << TWEA);        //next step: data byte will be received and ACK returned
    } else {
      TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE);        //next step: data byte will be received and NOACK returned
    }

    break;



    /******************************************
    \internal
    SLAVE SEND mode
    *******************************************/


    //SLA + Read Bit received and ACK was returned
  case I2C_SLAR_ACK_RCVD:
    //or Arbitration lost as Master (SLAR), SLAR received and ACK was returned
  case I2C_SLAR_ACK_ARB_LOST:

    address_counter = 0;                      //set adress counter to 0
    *number_bytes = 0;                            //set number of transferred bytes to 0

    //calculate current address
    current_address = data[0] + 1;

    //valid address, also for next byte
    if (current_address < (I2C_SLAVE_BUFFER_LENGTH - 1)) {

      TWDR = data[current_address];
      *number_bytes = *number_bytes + 1;
      address_counter++;
      //slave mode aborted or not?
      if (command == I2C_SLAVE_MODE_ABORT) {
        TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE);        //next step: data byte will be transmitted and NOACK returned
      } else {
        TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE) | (1 << TWEA);        //next step: data byte will be transmitted and ACK received
      }

      //current address is valid, but last byte of buffer
    } else if (current_address < I2C_SLAVE_BUFFER_LENGTH) {

      TWDR = data[current_address];
      *number_bytes = *number_bytes + 1;
      address_counter++;
      TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE);        //next step: data byte will be transmitted and NOACK received/returned

      //current address is invalid
    } else {

      TWDR = I2C_REGISTER_ERROR_VALUE;                //send error value to indicate an invalid address
      TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE);        //next step: data byte will be transmitted and NOACK received/returned

    }

    break;


    //Data transmitted, ACK received
  case I2C_DATA_ACK_TRANS:

    //calculate current address
    current_address = data[0] + address_counter + 1;

    //valid address, also for next byte
    if (current_address < (I2C_SLAVE_BUFFER_LENGTH - 1)) {

      TWDR = data[current_address];
      *number_bytes = *number_bytes + 1;
      address_counter++;
      //slave mode aborted or not?
      if (command == I2C_SLAVE_MODE_ABORT) {
        TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE);        //next step: data byte will be transmitted and NOACK returned
      } else {
        TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE) | (1 << TWEA);        //next step: data byte will be transmitted and ACK received
      }

      //current address is valid, but last byte of buffer
    } else if (current_address < I2C_SLAVE_BUFFER_LENGTH) {

      TWDR = data[current_address];
      *number_bytes = *number_bytes + 1;
      address_counter++;
      TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE);        //next step: data byte will be received and NOACK received/returned

      //current address is invalid
    } else {

      TWDR = I2C_REGISTER_ERROR_VALUE;                //send error value to indicate an invalid address
      TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE);        //next step: data byte will be received and NOACK received/returned

    }

    break;


    /******************************************
    \internal
    END OF TRANSMISSION:
    *******************************************/

    //Data transmitted, NOACK received
  case I2C_DATA_NOACK_TRANS:
    //Last Data Byte transmitted, ACK received
  case I2C_LAST_DATA_ACK_TRANS:

    //slave mode continued or not?
    if (command == I2C_SLAVE_MODE_CONTINUE) {
      TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE) | (1 << TWEA);        //next step: ACK will be returned for new request
    } else {
      TWCR = (1 << TWINT) | (1 << TWEN) | (TWIE_tmp << TWIE);        //next step: NOACK will be returned for new request
    }

    break;



  default:

#if RTC_LCD_DEBUG == 1
    i2c_error_counter++;
    lcd_set_cursor(8, 2);
    itoa(status, tmp_char, 16);
    lcd_send_string(tmp_char, FALSE);
    lcd_send_string(" ", FALSE);
    itoa(i2c_error_counter, tmp_char, 10);
    lcd_send_string(tmp_char, FALSE);
#endif

#if I2C_RESET_BUS_ON_ERROR == 1
    if ((TWCR & (1 << TWSTO)) > 0) {
      TWCR = 0;
    }
#endif

    //slave mode continued or not?
    if (command == I2C_SLAVE_MODE_CONTINUE) {
      TWCR = (1 << TWINT) | (1 << TWSTO) | (1 << TWEN) | (TWIE_tmp << TWIE) | (1 << TWEA);        		 //go into non-addressed mode and end slave mode
    } else {
      TWCR = (1 << TWINT) | (1 << TWSTO) | (1 << TWEN) | (TWIE_tmp << TWIE);        		 //go into non-addressed mode
    }

    return I2C_IO_BUS_ERROR;


  }


  return status;

}
