/****************************************************************************/
/**
* \file i2c_rtc_control_operation.c
*
* \brief Contains the process flow for the I2C I/O of the several
*        data transmission tasks for rtc_control.
*
* \author   Michael Skutek
* \date	    22.02.2013
* \version  0.2
*
* Platform:    AVR ATmega (tested on ATmega168)
*
* Include Files:
*       - rtc_control.h:        all global definitions
*       - i2c.h:                I2C related functions
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program. If not, see
* <http://www.gnu.org/licenses/>.
*
****************************************************************************/


/******************************************
	\internal
	Include files
*******************************************/

#include <stdlib.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#include "basic_definitions.h"
#include "rtc_control.h"
#include "i2c.h"
#include "dcf77.h"
#include "saa1064.h"
#include "i2c_rtc_control_operation.h"

#if RTC_LCD_DEBUG == 1 || RTC_LCD_TIMEDEBUG == 1
#include "lcd.h"
#endif






/******************************************
	\internal
	Pre-defined Strings
	(saved in program memory)
*******************************************/

#if RTC_LCD_DEBUG == 1 || RTC_LCD_TIMEDEBUG == 1
static const char mode_12h_MSG[] PROGMEM = "12-h-Modus";
static const char mode_24h_MSG[] PROGMEM = "24-h-Modus";
static const char no_battery_MSG[] PROGMEM = "Batterie-Fehler!";
static const char battery_MSG[] PROGMEM = "Batterie ok.";
static const char SAA1064_Err1_MSG[] PROGMEM = "SAA1064 Err1";
static const char SAA1064_Err2_MSG[] PROGMEM = "SAA1064 Err2";
static const char SAA1064_Err3_MSG[] PROGMEM = "SAA1064 Err3";
#endif






/****************************************************************************/
/**
* This function checks the DS1307 chip for it's operational state.
* Therefore it reads the oscillator state - if the oscillator is running,
* the DS1307 seems to provide the correct time data (via backup battery).
* If the oscillator is not running, the function switches it on. \n
*
* \param [in] rtc_data             array for the data transfer to the
*                                  DS1307 chip, it will be filled by this
*                                  function an does not require any data
*                                  at the function call
*
*
* \see I2C_MASTER_BUFFER_LENGTH
*
* \date
* - Created 26.01.2013
* - Modified  26.01.2013
*
*
* \author
*   Michael Skutek, michael@skutek.de
*/
/****************************************************************************/

void rtc_setup_ds1307(uint8_t *rtc_data)
{

  //for I2C communication
  uint8_t i2c_retry_counter;

  //other
  uint8_t return_value;


  //check for acivated clock oscillator which indicates an battery-powered state
  //set register address
  rtc_data[0] = DS1307_I2C_ADRESS;       //slave address+Write-Bit
  rtc_data[1] = 0x00;

  //I2C communication
  i2c_retry_counter = I2C_MAX_RETRYS;
  return_value = I2C_IO_ERROR;
  while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
    return_value = i2c_master_data_io(rtc_data, 2, FALSE);
    i2c_retry_counter--;
    _delay_us(I2C_RETRY_WAIT_US);
  }


  if (return_value == I2C_IO_SUCC) {

    rtc_data[0] = DS1307_I2C_ADRESS | 0x01;     //slave address+Read-Bit

    //I2C communication
    i2c_retry_counter = I2C_MAX_RETRYS;
    return_value = I2C_IO_ERROR;
    while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
      return_value = i2c_master_data_io(rtc_data, 4, FALSE);
      i2c_retry_counter--;
      _delay_us(I2C_RETRY_WAIT_US);
    }

    if (return_value == I2C_IO_SUCC) {
#if RTC_LCD_DEBUG == 1 || RTC_LCD_TIMEDEBUG == 1
      lcd_clear();
      if ((rtc_data[3] & 0x10) == 1) {          //12/24 hours mode
        lcd_send_string(mode_12h_MSG, TRUE);
      } else {
        lcd_send_string(mode_24h_MSG, TRUE);
      }
      lcd_set_cursor(0, 1);
#endif
      if ((rtc_data[1] & 0x80) == 0x80) {      //if set, oscillator is switched off --> no battery
#if RTC_LCD_DEBUG == 1 || RTC_LCD_TIMEDEBUG == 1
        lcd_send_string(no_battery_MSG, TRUE);
#endif

        //setup register data
        rtc_data[0] = DS1307_I2C_ADRESS;            //slave address + Write-Bit
        rtc_data[1] = 0x00;                         //DS1307 register address
        rtc_data[2] = 0x00;                         //second register
        rtc_data[3] = 0x00;                         //minute register
        rtc_data[4] = 0x00;                         //hour register
        rtc_data[5] = 0x01;                         //day register
        rtc_data[6] = 0x01;                         //date register
        rtc_data[7] = 0x01;                         //month register
        rtc_data[8] = 0x00;                         //year register
        rtc_data[9] = 0x10;                         //control register

        //I2C communication
        i2c_retry_counter = I2C_MAX_RETRYS;
        return_value = I2C_IO_ERROR;
        while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
          return_value = i2c_master_data_io(rtc_data, 10, FALSE);
          i2c_retry_counter--;
          _delay_us(I2C_RETRY_WAIT_US);
        }

      } else {
        //setup register data
        rtc_data[0] = DS1307_I2C_ADRESS;            //slave address + Write-Bit
        rtc_data[1] = 0x07;                         //DS1307 register address
        rtc_data[2] = 0x10;                         //control register

        //I2C communication
        i2c_retry_counter = I2C_MAX_RETRYS;
        return_value = I2C_IO_ERROR;
        while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
          return_value = i2c_master_data_io(rtc_data, 3, FALSE);
          i2c_retry_counter--;
          _delay_us(I2C_RETRY_WAIT_US);
        }

#if RTC_LCD_DEBUG == 1 || RTC_LCD_TIMEDEBUG == 1
        lcd_send_string(battery_MSG, TRUE);
#endif
      }
      _delay_ms(2000);           //wait for x ms
    }
  }

}




/****************************************************************************/
/**
* This function reads the status registers of the SAA1064 ICs (if enabled)
* and sets upt the initial parameters of the SAA1064 (LED current). \n
*
* \param [in] saa1064_data         array for the data transfer to the
*                                  SAA1064 chip, it will be filled by this
*                                  function an does not require any data
*                                  at the function call
*
* \see I2C_SAA1064_NUMBER_REG
*
* \date
* - Created 26.01.2013
* - Modified  26.01.2013
*
*
* \author
*   Michael Skutek, michael@skutek.de
*/
/****************************************************************************/

void rtc_setup_saa1064(uint8_t *saa1064_data)
{


#if RTC_LCD_DEBUG == 1

  uint8_t return_value;

  ///check all three SAA1064 ICs

  saa1064_data[0] = 0x00;
  saa1064_data[1] = 0x00;
  saa1064_data[2] = 0x00;
  saa1064_data[3] = 0x00;
  saa1064_data[4] = 0x00;
  saa1064_data[5] = 0x00;
  saa1064_data[6] = 0x00;
  return_value = saa1064_read_status(I2C_SAA1064_ADDRESS1, saa1064_data);

  if (return_value != SAA1064_IO_SUCCESS) {
    lcd_clear();
    lcd_send_string(SAA1064_Err1_MSG, TRUE);
    _delay_ms(500);
    _delay_ms(500);
    _delay_ms(500);
    _delay_ms(500);
  }

  saa1064_data[0] = 0x00;
  saa1064_data[1] = 0x00;
  saa1064_data[2] = 0x00;
  saa1064_data[3] = 0x00;
  saa1064_data[4] = 0x00;
  saa1064_data[5] = 0x00;
  saa1064_data[6] = 0x00;
  return_value = saa1064_read_status(I2C_SAA1064_ADDRESS2, saa1064_data);

  if (return_value != SAA1064_IO_SUCCESS) {
    lcd_clear();
    lcd_send_string(SAA1064_Err2_MSG, TRUE);
    _delay_ms(500);
    _delay_ms(500);
    _delay_ms(500);
    _delay_ms(500);
  }

  saa1064_data[0] = 0x00;
  saa1064_data[1] = 0x00;
  saa1064_data[2] = 0x00;
  saa1064_data[3] = 0x00;
  saa1064_data[4] = 0x00;
  saa1064_data[5] = 0x00;
  saa1064_data[6] = 0x00;
  return_value = saa1064_read_status(I2C_SAA1064_ADDRESS3, saa1064_data);

  if (return_value != SAA1064_IO_SUCCESS) {
    lcd_clear();
    lcd_send_string(SAA1064_Err3_MSG, TRUE);
    _delay_ms(500);
    _delay_ms(500);
    _delay_ms(500);
    _delay_ms(500);
  }

#else       //RTC_LCD_DEBUG == 1

  UNUSED(saa1064_data);

#endif      //RTC_LCD_DEBUG == 1



  ///SAA1064 initialization

  saa1064_init(I2C_SAA1064_ADDRESS1, SAA1064_STATIC_MODE, 0xff);
  saa1064_init(I2C_SAA1064_ADDRESS2, SAA1064_STATIC_MODE, 0xff);
  saa1064_init(I2C_SAA1064_ADDRESS3, SAA1064_STATIC_MODE, 0xff);




}



/****************************************************************************/
/**
* This function shows the current time via SAA1064. It updates the time data
* only if the second value has changed to avoid overstressing the I2C bus.\n
* Depending on the day and night time cycles, it uses two different LED
* current values (dimmed night mode). Additionally to that, it signalizes
* an synchronized DCF77 signal by switching on the decimal point of the
* last digit.
*
* \param [in] current_hour         current hour value
* \param [in] current_minute       current minute value
* \param [in] current_second       current second value
* \param [in] dcf_status           current status of DCF77 synchronization
* \param [in] update_all           if > 0, all values will immediately be updated,
*                                  otherwise the update happens only for changed
*                                  values
*
*
* \see DCF77 state definition in file dcf77.h
*
* \date
* - Created 03.02.2013
* - Modified  08.02.2013
*
*
* \author
*   Michael Skutek, michael@skutek.de
*/
/****************************************************************************/

void rtc_show_time_saa1064(uint8_t current_hour, uint8_t current_minute, uint8_t current_second,
                           uint8_t dcf_status, uint8_t update_all)
{


  static uint8_t prev_second = 0;             // saves the previous second value to detect changes
  static uint8_t prev_minute = 0;             // saves the previous minute value to detect changes
  static uint8_t prev_hour = 0;               // saves the previous hour value to detect changes
  static uint8_t startup = TRUE;              // indicates the first call

  static uint8_t ff_mem = 0;                  //memory for flipflop
  static uint32_t tmp_time = 0;               //temporary value for calculating the current configuration


  //reset startup flag
  if (update_all > 0) {
    startup = TRUE;
  }


  //determine the LED current - compare time with configuration
  tmp_time = ((uint32_t) current_hour * (uint32_t) 10000) +
             ((uint32_t) current_minute * (uint32_t) 100) +
             (uint32_t) current_second;


  //update all segments at the same time
  if ((tmp_time == (uint32_t) LED_LOW_CURRENT_START_TIME) || (tmp_time == (uint32_t) LED_HIGH_CURRENT_START_TIME)) {
    startup =  TRUE;
  }

  //use different modes for night and day --> night mode
  if ((tmp_time >= (uint32_t) LED_LOW_CURRENT_START_TIME) || (tmp_time < (uint32_t) LED_HIGH_CURRENT_START_TIME)) {

    //do only updates for hours and minutesand seconds, if these changed
    if ((current_hour != prev_hour) || (startup == TRUE)) {
      saa1064_send_digits(I2C_SAA1064_ADDRESS1, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT01, current_hour, 0x01, 0);
      _delay_ms(400.0);  //creates a delayed update of the later digits
    }

    if ((current_minute != prev_minute) || (startup == TRUE)) {
      saa1064_send_digits(I2C_SAA1064_ADDRESS2, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT01, current_minute, 0x00, 0);
      //_delay_ms(400.0);  //creates a delayed update of the later digits
    }

    if ((current_second != prev_second) || (startup == TRUE)) {
      if (dcf_status == DCF77_TIMESYNC_COMPLETED) {
        saa1064_send_digits(I2C_SAA1064_ADDRESS3, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT01, current_second, 0x01, 1);
      } else {
        saa1064_send_digits(I2C_SAA1064_ADDRESS3, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT01, current_second, 0x00, 1);
      }

    }

    //use different modes for night and day --> day mode
  } else {

    //do only updates for hours and minutes and seconds, if these changed
    if ((current_hour != prev_hour) || (startup == TRUE)) {
      saa1064_send_digits(I2C_SAA1064_ADDRESS1, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_hour, 0x01, 0);
      _delay_ms(400.0);  //creates a delayed update of the later digits
    }

    if ((current_minute != prev_minute) || (startup == TRUE)) {
      saa1064_send_digits(I2C_SAA1064_ADDRESS2, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_minute, 0x00, 0);
      _delay_ms(400.0);  //creates a delayed update of the later digits
    }

    if ((current_second != prev_second) || (startup == TRUE)) {
      if (dcf_status != DCF77_TIMESYNC_COMPLETED) {
        if (ff_mem > 0) {
          saa1064_send_digits(I2C_SAA1064_ADDRESS3, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_second, 0x02, 0);
          ff_mem = 0;
        } else {
          saa1064_send_digits(I2C_SAA1064_ADDRESS3, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_second, 0x01, 0);
          ff_mem = 1;
        }
      } else {
        saa1064_send_digits(I2C_SAA1064_ADDRESS3, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_second, 0x01, 0);
      }

    }

  }

  //save the current value to detect changes
  prev_second = current_second;
  prev_minute = current_minute;
  prev_hour = current_hour;
  startup = FALSE;

}




/****************************************************************************/
/**
* This function shows the current date via SAA1064. It switches the date
* for each function call from day/month to year and back. Therefore this
* function should be called for 4 times to go through all the states.\n
* Depending on the day and night time cycles, it uses two different LED
* current values (dimmed night mode) and two different types of date
* information changing.
*
* \param [in] current_day          current day value
* \param [in] current_month        current month value
* \param [in] current_year         current year value
* \param [in] current_hour         current hour value (for day/night mode)
* \param [in] current_minute       current minute value (for day/night mode)
* \param [in] current_second       current second value (for day/night mode)
* \param [in] reset                if > 0, the functions resets the internal
*                                  state and clears all digits
*
* \date
* - Created 25.02.2013
* - Modified  25.02.2013
*
*
* \author
*   Michael Skutek, michael@skutek.de
*/
/****************************************************************************/

void rtc_show_date_saa1064(uint8_t current_day, uint8_t current_month, uint16_t current_year, uint8_t current_hour,
                           uint8_t current_minute, uint8_t current_second, uint8_t reset)
{

  static uint8_t prev_second = 0;             // saves the previous second value to detect changes
  static uint8_t startup = TRUE;              // indicates the first call
  static uint8_t mode = 0;

  static uint32_t tmp_time = 0;               //temporary value for calculating the current configuration


  //determine the LED current - compare time with configuration
  tmp_time = ((uint32_t) current_hour * (uint32_t) 10000) +
             ((uint32_t) current_minute * (uint32_t) 100) +
             (uint32_t) current_second;

  //update all segments at the same time
  if ((tmp_time == (uint32_t) LED_LOW_CURRENT_START_TIME) || (tmp_time == (uint32_t) LED_HIGH_CURRENT_START_TIME)) {
    mode = 0;
    startup =  TRUE;
  }

  // reset the mode memory, if requested
  if (reset > 0) {

    mode = 0;
    saa1064_send_digits(I2C_SAA1064_ADDRESS1, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_day, 0x00, 1);    //clear
    saa1064_send_digits(I2C_SAA1064_ADDRESS2, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_month, 0x00, 1);  //clear
    saa1064_send_digits(I2C_SAA1064_ADDRESS3, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_year, 0x00, 1);   //clear

  } else {


    if ((current_second != prev_second) || (startup == TRUE)) {

      //use different modes for night and day --> night mode
      if ((tmp_time >= (uint32_t) LED_LOW_CURRENT_START_TIME) || (tmp_time < (uint32_t) LED_HIGH_CURRENT_START_TIME)) {

        if (mode == 0) {

          saa1064_send_digits(I2C_SAA1064_ADDRESS1, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT01, current_day, 0x01, 0);
          saa1064_send_digits(I2C_SAA1064_ADDRESS2, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT01, current_month, 0x01, 0);
          saa1064_send_digits(I2C_SAA1064_ADDRESS3, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT01, current_year, 0x00, 1);
          _delay_ms(800.0);  //creates a delayed update of the later digits

        } else if (mode == 1) {

          saa1064_send_digits(I2C_SAA1064_ADDRESS1, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT01, current_day, 0x00, 1);
          saa1064_send_digits(I2C_SAA1064_ADDRESS2, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT01, current_month, 0x00, 1);
          saa1064_send_digits(I2C_SAA1064_ADDRESS3, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT01, current_year, 0x00, 1);
          _delay_ms(800.0);  //creates a delayed update of the later digits

        } else if (mode == 2) {

          saa1064_send_digits(I2C_SAA1064_ADDRESS1, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT01, current_day, 0x00, 1);
          saa1064_send_digits(I2C_SAA1064_ADDRESS2, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT01, (current_year + RTC_BASE_YEAR) / 100, 0x00, 0);
          saa1064_send_digits(I2C_SAA1064_ADDRESS3, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT01, current_year, 0x00, 0);

        } else {

          saa1064_send_digits(I2C_SAA1064_ADDRESS1, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT01, current_day, 0x00, 1);
          saa1064_send_digits(I2C_SAA1064_ADDRESS2, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT01, (current_year + RTC_BASE_YEAR) / 100, 0x00, 1);
          saa1064_send_digits(I2C_SAA1064_ADDRESS3, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT01, current_year, 0x00, 1);
          _delay_ms(800.0);  //creates a delayed update of the later digits

        }

        if (mode < 3) {
          mode++;
        } else {
          mode = 0;
        }


        //use different modes for night and day --> day mode
      } else {

        if (mode == 0) {

          saa1064_send_digits(I2C_SAA1064_ADDRESS1, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_day, 0x01, 0);
          _delay_ms(400.0);  //creates a delayed update of the later digits
          saa1064_send_digits(I2C_SAA1064_ADDRESS2, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_month, 0x01, 0);
          _delay_ms(400.0);  //creates a delayed update of the later digits
          saa1064_send_digits(I2C_SAA1064_ADDRESS3, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_year, 0x00, 1);

        } else if (mode == 1) {

          saa1064_send_digits(I2C_SAA1064_ADDRESS1, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_day, 0x00, 1);
          _delay_ms(400.0);  //creates a delayed update of the later digits
          saa1064_send_digits(I2C_SAA1064_ADDRESS2, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_month, 0x00, 1);
          _delay_ms(400.0);  //creates a delayed update of the later digits
          saa1064_send_digits(I2C_SAA1064_ADDRESS3, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_year, 0x00, 1);

        } else if (mode == 2) {

          saa1064_send_digits(I2C_SAA1064_ADDRESS1, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_day, 0x00, 1);
          // _delay_ms(400.0);  //creates a delayed update of the later digits
          saa1064_send_digits(I2C_SAA1064_ADDRESS2, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, (current_year + RTC_BASE_YEAR) / 100, 0x00, 0);
          _delay_ms(400.0);  //creates a delayed update of the later digits
          saa1064_send_digits(I2C_SAA1064_ADDRESS3, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_year, 0x00, 0);

        } else {

          saa1064_send_digits(I2C_SAA1064_ADDRESS1, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_day, 0x00, 1);
          //_delay_ms(400.0);  //creates a delayed update of the later digits
          saa1064_send_digits(I2C_SAA1064_ADDRESS2, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, (current_year + RTC_BASE_YEAR) / 100, 0x00, 1);
          _delay_ms(400.0);  //creates a delayed update of the later digits
          saa1064_send_digits(I2C_SAA1064_ADDRESS3, SAA1064_STATIC_MODE, SAA1064_LED_CURRENT03, current_year, 0x00, 1);

        }

        if (mode < 3) {
          mode++;
        } else {
          mode = 0;
        }

      }


    }

  }

  prev_second = current_second;
  startup = FALSE;



}
