/****************************************************************************/
/**
* \file isr.c
*
* \brief Contains the interrupt service routines for the real time clock module.
*
* \author   Michael Skutek
* \date	    20.03.2013
* \version  0.3
*
* Platform:    AVR ATmega (tested on ATmega168)
*
* Include Files:
*       - avr/io.h:             port accessing (part of avr-gcc)
*       - avr/interrupt.h:      interrupt related (part of avr-gcc)
*       - rtc_control.h:        all global definitions
*       - i2c.h:                I2C related functions
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program. If not, see
* <http://www.gnu.org/licenses/>.
*
****************************************************************************/


/******************************************
	\internal
	Include files
*******************************************/

#include <avr/io.h>
#include <avr/interrupt.h>

#include "rtc_control.h"
#include "port_definitions.h"
#include "i2c.h"
#include "dcf77.h"

#if RTC_LCD_DEBUG == 1
#include <stdlib.h>
#include "lcd.h"
#endif


/******************************************
	\internal
	Global variables
*******************************************/

extern volatile uint8_t time_scale;                   // counter triggered by timer

extern volatile uint8_t i2c_slave_data_arrived;       // indicates new arrived data from I2C master in buffer
extern volatile uint8_t slave_number_bytes;           // number of bytes sent by I2C master
extern volatile uint8_t i2c_slave_mode_blocked;       // device is blocked an can not receive any data from I2C master
extern volatile uint8_t ctrl_data[10];                // buffer for data I/O with an external master (via interrupt)

extern volatile dcf77_config_data dcf77_config;                //!< contains the configuration for the DCF77 module
extern volatile real_time_data dcf77_time;                     //!< contains the real time data from DCF77 module



/****************************************************************************/
/**
* Timer Interrupt Service Routine. It increases a counter value by 1 (10 ms
* cycle time).
* Additionally, it calls the DCF77 routines to get a synchronization. This
* interrupt based signal processing ensures an operation independly from
* other functions which have delays or runtimes longer than 10 ms.\n
*
* \see RTC_OCR0A in rtc_control.h
*
* \date
* - Created 28.09.2011
* - Modified  24.03.2013
*
* \author
*   Michael Skutek, michael@skutek.de
*/
/****************************************************************************/

ISR(TIMER0_COMPA_vect) {

  //set time scale
  if (time_scale >= MAX_TIME_SCALE) {
    time_scale = 0;
  } else {
    time_scale++;
  }

  //update DCF77 sync
  if (dcf77_config.dcf_enabled > 0) {

    dcf77_config.current_dcf_signal = (ACSR >> ACO) & 0x01;

    if (dcf77_config.current_dcf_signal == 1) {                         //comparator input below bandgap reference
      DCF_LED_PORT = DCF_LED_PORT & ~(1 << DCF_LED_ADDR);
    } else {
      DCF_LED_PORT = DCF_LED_PORT | (1 << DCF_LED_ADDR);
    }

    //sync only if sync is not completed
    if (dcf77_config.dcf_state != DCF77_TIMESYNC_COMPLETED) {
      dcf77_config.dcf_state = dcf77_sync(dcf77_config.current_dcf_signal, DCF77_ACTIVE_LOW,
                                          &dcf77_time, DCF77_SAMPLE_TIME, DCF77_GET_TIME);

    }
  }


}


/****************************************************************************/
/**
* TWI (I2C) Interrupt Service Routine. It handles the I2C data I/O during the
* slave mode and sets the "slave blocked" indication (i2c_slave_mode_blocked).
*
*
* \date
* - Created 28.09.2011
* - Modified  28.09.2011
*
* \author
*   Michael Skutek, michael@skutek.de
*/
/****************************************************************************/


ISR(TWI_vect) {

  uint8_t status;
#if RTC_LCD_DEBUG == 1
  static char tmp_char[10];
#endif

  status = i2c_slave_data_io(ctrl_data, &slave_number_bytes, I2C_SLAVE_MODE_CONTINUE);

  if ((status == I2C_STO_REPSTART_COND) ||
      (status == I2C_DATA_NOACK_RCVD) ||
      (status == I2C_GCA_DATA_NOACK_RCVD)) {
    i2c_slave_mode_blocked = FALSE;            //Transmission completed
    if (slave_number_bytes > 1) {   //more than only register address set?
      i2c_slave_data_arrived = TRUE;
    }
  } else if (status == I2C_DATA_NOACK_TRANS || status == I2C_LAST_DATA_ACK_TRANS) {
    i2c_slave_mode_blocked = FALSE;            //Transmission completed
  } else if ((status == I2C_SLAW_ACK_RCVD) ||
             (status == I2C_SLAW_ACK_ARB_LOST) ||
             (status == I2C_GCA_ACK_RCVD) ||
             (status == I2C_GCA_ACK_ARB_LOST) ||
             (status == I2C_DATA_ACK_RCVD) ||
             (status == I2C_GCA_DATA_ACK_RCVD) ||
             (status == I2C_SLAR_ACK_RCVD) ||
             (status == I2C_SLAR_ACK_ARB_LOST) ||
             (status == I2C_DATA_ACK_TRANS)) {
    i2c_slave_mode_blocked = TRUE;             //Transmission NOT completed
  } else if (status == I2C_IO_BUS_ERROR) {
    i2c_slave_mode_blocked = FALSE;            //Transmission aborted or no transmission
  } else {
#if RTC_LCD_DEBUG == 1
    lcd_set_cursor(14, 3);
    lcd_send_string("!R", FALSE);
    itoa(status, tmp_char, 16);
    lcd_send_string(tmp_char, FALSE);
#endif
  }

}

