/****************************************************************************/
/**
* \file main.c
*
* \brief Contains the main routine for the real time clock module.
*
* \author   Michael Skutek
* \date	    20.03.2013
* \version  0.3
*
* Platform:    AVR ATmega (tested on ATmega168)
*
* Include Files:
*       - avr/io.h:             port accessing (part of avr-gcc)
*       - dcf77.h:              all internal definitions
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program. If not, see
* <http://www.gnu.org/licenses/>.
*
****************************************************************************/


/******************************************
	\internal
	Include files
*******************************************/

#include <avr/io.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include <math.h>

#include "rtc_control.h"
#include "port_definitions.h"
#include "i2c.h"
#include "dcf77.h"
#include "i2c_rtc_control_operation.h"

#if RTC_LCD_DEBUG == 1 || RTC_LCD_TIMEDEBUG == 1
#include "lcd.h"
#endif

#if RTC_SAA1064_OUTPUT == 1
#include "saa1064.h"
#endif // RTC_SAA1064_OUTPUT



/******************************************
	\internal
	Definitions
*******************************************/

//Definition for Frequency Setting
#ifndef F_CPU
#warning "F_CPU was not defined! It will now be set to 20000000UL."
#define F_CPU 20000000UL                                //!< Controller Clock Cycle [Hz]
#endif


/******************************************
	\internal
	Global variables
*******************************************/

volatile uint8_t time_scale;                            //!< counter triggered by timer
volatile uint8_t prev_time_scale;                       //!< previous counter value (triggered by timer)

volatile uint8_t i2c_slave_data_arrived;                //!< indicates new arrived data from I2C master in buffer
volatile uint8_t slave_number_bytes;                    //!< number of bytes sent by I2C master
volatile uint8_t i2c_slave_mode_blocked;                //!< device is blocked an can not receive any data from I2C master
volatile uint8_t ctrl_data[I2C_SLAVE_BUFFER_LENGTH];    //!< buffer for data I/O with an external master (via interrupt)

volatile dcf77_config_data dcf77_config;                //!< contains the configuration for the DCF77 module
volatile real_time_data dcf77_time;                     //!< contains the real time data from DCF77 module


#if RTC_LCD_DEBUG == 1 || RTC_LCD_TIMEDEBUG == 1
/******************************************
	\internal
	Pre-defined Strings
	(saved in program memory)
*******************************************/
static const char slave_mode_error_MSG[] PROGMEM = "INTERNAL ERROR (SLA)";
static const char clear_line_MSG[] PROGMEM = "                    ";
#endif




/****************************************************************************/
/**
* Contains the main function - the initialization of all ports, functions
* and the main loop: \n
* <ol>
* <li>Resetting of all pins of all ports (to input pins with pull-up resistors).</li>
* <li>Initialization of LCD pins (if RTC_LCD_DEBUG or RTC_LCD_TIMEDEBUG is set to 1).</li>
* <li>Initialization of Comparator Pin and Comparator itself (input from DCF77 module).</li>
* <li>Initialization of the pin for the DCF77 indicator LED.</li>
* <li>Initialization of I2C pins.</li>
* <li>Initialization of LCD (if RTC_LCD_DEBUG or RTC_LCD_TIMEDEBUG is set to 1).</li>
* <li>Initialization of Timer and the respective Interrupt.</li>
* <li>Initialization of I2C and configuration as I2C slave.</li>
* <li>Initialization of internal data structures.</li>
* <li>Read current state of DS1307 RTC chip and set it up if it has no battery powered time information.</li>
* <li>Read current state of SAA1064 LED 7 segment driver chip and start a LED checkup/demo.</li>
* <li>Start DCF77 module (if enabled) and start a new synchronization cycle.</li>
* <li>Main Loop:</li>
* <ol>
* <li>Check for arrived data from an external I2C master (get time command or set config command) and
*     and perform the requested operation (using the control registers ctrl_data). The following operations
*     (sending or receiving data to or from DS1307 RTC chip) will be performed as I2C master.</li>
* <li>If the DCF77 module is enabled, read in the current signal value.</li>
* <li>If the DCF77 module is enabled, start an new synchronization cycle, when an update cycle has
*      reached (every DCF77_UPDATE_CYCLE minutes). During an update cycle, the function dcf77_sync()
*      is periodically called via Interrupt Service Routine ISR(TIMER0_COMPA_vect) and, depending on the internal state
*      (return value), the respective time information is saved into the dcf77_time structure
*      (and written to the DS1307 RTC chip, except of the seconds). After receiving the last data item
*      (year), the synchronization cycle is finished and the received data (including the seconds)
*      will be written to the DS1307 RTC chip.</li>
* <li>Read time data from DS1307 RTC chip and print it on the LCD and or LED, if the LCD and/or LED
*     output is enabled.</li>
* <li>Save current configuration and time data into control registers (ctrl_data). These registers
*     are the I/O registers for I2C communication as slave.</li>
* </ol>
* </ol>
*
*
* \date
* - Created 05.08.2010
* - Modified  20.03.2013
*
*
* \see dcf77.h
* \see lcd.h
* \see port_definitions.h
*
*
* \author
*   Michael Skutek, michael@skutek.de
*/
/****************************************************************************/

int main(void)
{


#if RTC_SAA1064_OUTPUT == 1
  //for LED output (SAA1064)
  static uint8_t saa1064_data[I2C_SAA1064_NUMBER_REG];
#endif // RTC_SAA1064_OUTPUT

  //for real time clock
  static uint8_t rtc_data[I2C_MASTER_BUFFER_LENGTH];      // I2C data buffer for data transmission from or to DS1307 RTC chip
  static real_time_data real_time;                        // contains the real time data from DS1307 RTC chip



  //timer and bus-related
#if RTC_LCD_DEBUG == 1
  uint8_t timer_value_start;
#endif
  uint8_t i2c_retry_counter;

  uint8_t prev_dcf77_state = DCF77_STARTSYNC_SEARCH01;

  //other
  uint8_t return_value;

#if RTC_LCD_DEBUG == 1 || RTC_LCD_TIMEDEBUG == 1
  static char tmp_char[40];
  uint8_t ff_mem = 0;
#endif



  /***************************************************************************
  \internal
  port and pin configuration
  ***************************************************************************/

  //enable pull-ups for all pins an set them to input port (default state)
  PORTB = 0xff;
  DDRB = 0x00;
  PORTC = 0xff;
  DDRC = 0x00;
  PORTD = 0xff;
  DDRD = 0x00;

#if RTC_LCD_DEBUG == 1 || RTC_LCD_TIMEDEBUG == 1
  //pin initialization for LCD module
  lcd_pin_init();
  lcd_blpin_init();
#endif

  //pin initialization for DCF77 I/O
  COMP_PORT = COMP_PORT & ~(1 << COMP_DATA);
  COMP_DDR = COMP_DDR & ~(1 << COMP_DATA);
  PON_PORT = PON_PORT & ~(1 << PON_ADDR);
  PON_DDR = PON_DDR | (1 << PON_ADDR);

  //Comparator configuration for COMP_DATA
  ADMUX = 0xC2;                    //ADC input Pin 2
  ADCSRA = 0x00;                   //switch ADC off
  ADCSRB = 0x40;                   //use ADC-Pin as input (multiplex)
  ACSR = ACSR | (1 << ACBG);     //enable internal bandgap reference
  ACSR = ACSR & ~(1 << ACD) & ~(1 << ACIE) & ~(1 << ACIC);   //switch on Comparator, disable interrupt, disable input capture

  //pin initialization for LED
  DCF_LED_PORT = DCF_LED_PORT & ~(1 << DCF_LED_ADDR);
  DCF_LED_DDR = DCF_LED_DDR | (1 << DCF_LED_ADDR);


  //pin initialization for I2C
  i2c_pin_init();


  /***************************************************************************
  \internal
  Initialization of sub-modules and interrupts
  ***************************************************************************/

#if RTC_LCD_DEBUG == 1 || RTC_LCD_TIMEDEBUG == 1
  //initialize LCD
  lcd_init();
#endif


  //set up Timer ISR
  TCCR0A = 0x02;          //CTC mode
  //TCCR0B = 0x0d;          //WGM02 set, Prescaler: 1024
  TCCR0B = 0x05;          //WGM02 set, Prescaler: 1024
  OCR0A = RTC_OCR0A;            //value for compare match (156 = 10 ms for 16 MHz clock, 195 = 10 ms for 20 MHz clock)
  TIMSK0 = TIMSK0 | (1 << OCIE0A); //enable interrupt for Timer A
  time_scale = 0;         //reset time scale (basis for some operations) [*10 ms]
  prev_time_scale = 0;    //previous time scale to detect changes


  //initialize I2C

  //set clock speed
  i2c_set_scl(F_CPU, I2C_SCL_FREQ);

  //set slave address
  TWAR = I2C_SLAVE_ADDRESS;

  //preset I2C slave data flag
  i2c_slave_data_arrived = FALSE;
  i2c_slave_mode_blocked = FALSE;

  //enable I2C slave operation mode
  TWCR = (1 << TWEN) | (1 << TWEA) | (1 << TWIE) | (1 << TWINT);


  //activate global Interrupts
  sei();


  /***************************************************************************
  \internal
  Initialization of data structures
  ***************************************************************************/

  real_time.current_hour = 0;
  real_time.current_minute = 0;
  real_time.current_second = 0;
  real_time.prev_second = 0;

  real_time.current_year = 0;
  real_time.current_month = 0;
  real_time.current_day = 0;
  real_time.current_weekday = 0;

  dcf77_time.current_hour = 0;
  dcf77_time.current_minute = 0;
  dcf77_time.current_second = 0;
  dcf77_time.prev_second = 0;

  dcf77_time.current_year = 0;
  dcf77_time.current_month = 0;
  dcf77_time.current_day = 0;
  dcf77_time.current_weekday = 0;


  //setup DCF signal memory and initial configuration
  dcf77_config.current_dcf_signal = 0;
  dcf77_config.dcf_enabled = DCF77_ENABLED_DEFAULT;


  /***************************************************************************
  \internal
  Setup RTC if not battery powered
  ***************************************************************************/

#if RTC_LCD_DEBUG == 1 || RTC_LCD_TIMEDEBUG == 1
  lcd_clear();
  lcd_send_string("RTC V0.3", FALSE);
#endif

  _delay_ms(500);    //wait for some time (maybe RTC chip needs some startup time...)

  //call DS1307 setup function
  rtc_setup_ds1307(rtc_data);


  /***************************************************************************
  \internal
  Setup SAA1064
  ***************************************************************************/

#if RTC_SAA1064_OUTPUT == 1
  //call SAA1064 setup function
  rtc_setup_saa1064(saa1064_data);
#endif // RTC_SAA1064_OUTPUT


  /***************************************************************************
  \internal
  Start DCF module and sync function
  ***************************************************************************/

  //start DCF module --> switch PON from 1 to 0 (inverted logic)
  if (dcf77_config.dcf_enabled > 0) {
    PON_PORT = PON_PORT | (1 << PON_ADDR);
  }


  //initialize the time sync function
  dcf77_config.dcf_state = dcf77_sync(dcf77_config.current_dcf_signal, DCF77_ACTIVE_LOW,
                                      &real_time, DCF77_SAMPLE_TIME, DCF77_RESET_TIME);


#if RTC_LCD_DEBUG == 1 || RTC_LCD_TIMEDEBUG == 1
  //clear LCD
  lcd_clear();
#endif




  /***************************************************************************
  \internal
  Main Loop
  ***************************************************************************/

  while (1) {


    if (prev_time_scale != time_scale) {

#if RTC_LCD_DEBUG == 1
      //get timer value to calculate computing time
      timer_value_start = TCNT0;
#endif

      /***************************************************************************
      \internal
      Check for I2C data from master
      ***************************************************************************/

      if (i2c_slave_data_arrived == TRUE) {

        //get data from register
        dcf77_config.dcf_enabled = (ctrl_data[1] >> 1) & 0x01;

        //set time command from master?
        if ((ctrl_data[1] & 0x80) == 0x80) {

          real_time.current_second = ctrl_data[2] & 0x3f;
          real_time.current_minute = (ctrl_data[2] >> 6) & 0x03;
          real_time.current_minute = real_time.current_minute | ((ctrl_data[3] & 0x0f) << 2);
          real_time.current_month = (ctrl_data[3] >> 4) & 0x0f;
          real_time.current_hour = ctrl_data[4] & 0x1f;
          real_time.current_weekday = (ctrl_data[4] >> 5) & 0x07;
          real_time.current_day = ctrl_data[5] & 0x1f;
          real_time.current_year = ctrl_data[6] & 0x7f;

          //write to RTC
          rtc_data[0] = DS1307_I2C_ADRESS;            //slave address + Write-Bit
          rtc_data[1] = 0x00;
          rtc_data[2] = ((real_time.current_second / 10) << 4) & 0x7f;      //*10 seconds
          rtc_data[2] = rtc_data[2] | ((real_time.current_second - ((real_time.current_second / 10) * 10)) & 0x0f);
          rtc_data[3] = (real_time.current_minute / 10) << 4;      //*10 minutes
          rtc_data[3] = rtc_data[3] | ((real_time.current_minute - ((real_time.current_minute / 10) * 10)) & 0x0f);
          rtc_data[4] = (real_time.current_hour / 10) << 4;      //*10 minutes
          rtc_data[4] = rtc_data[4] | ((real_time.current_hour - ((real_time.current_hour / 10) * 10)) & 0x0f);
          rtc_data[5] = real_time.current_weekday & 0x0f;
          rtc_data[6] = (real_time.current_day / 10) << 4;      // *10 minutes
          rtc_data[6] = rtc_data[6] | ((real_time.current_day - ((real_time.current_day / 10) * 10)) & 0x0f);
          rtc_data[7] = (real_time.current_month / 10) << 4;      // *10 minutes
          rtc_data[7] = rtc_data[7] | ((real_time.current_month - ((real_time.current_month / 10) * 10)) & 0x0f);
          rtc_data[8] = (real_time.current_year / 10) << 4;      // *10 minutes
          rtc_data[8] = rtc_data[8] | ((real_time.current_year - ((real_time.current_year / 10) * 10)) & 0x0f);

          //I2C communication
          i2c_retry_counter = I2C_MAX_RETRYS;
          return_value = I2C_IO_ERROR;
          while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
            return_value = i2c_master_data_io(rtc_data, 9, FALSE);
            i2c_retry_counter--;
            _delay_us(I2C_RETRY_WAIT_US);
          }

        }     //if ((ctrl_data[1] & 0x80) == 0x80) {...


#if RTC_LCD_DEBUG == 1
        lcd_set_cursor(0, 1);
        lcd_send_string("                ", FALSE);
        lcd_set_cursor(0, 1);
        itoa(ctrl_data[0], tmp_char, 10);
        lcd_send_string(tmp_char, FALSE);
        lcd_send_string(" ", FALSE);
        itoa(ctrl_data[1], tmp_char, 10);
        lcd_send_string(tmp_char, FALSE);
        lcd_send_string(" ", FALSE);
        itoa(ctrl_data[2], tmp_char, 10);
        lcd_send_string(tmp_char, FALSE);
        lcd_send_string(" ", FALSE);
        itoa(ctrl_data[3], tmp_char, 10);
        lcd_send_string(tmp_char, FALSE);
        lcd_send_string(" ", FALSE);
        itoa(ctrl_data[4], tmp_char, 10);
        lcd_send_string(tmp_char, FALSE);
        lcd_send_string(" ", FALSE);
        itoa(ctrl_data[5], tmp_char, 10);
        lcd_send_string(tmp_char, FALSE);
        lcd_send_string(" ", FALSE);
        itoa(ctrl_data[6], tmp_char, 10);
        lcd_send_string(tmp_char, FALSE);
#endif

        i2c_slave_data_arrived = FALSE;
      }



      /***************************************************************************
      \internal
      DCF Module Input
      ***************************************************************************/

      if (dcf77_config.dcf_enabled > 0) {

        /* dcf77_config.current_dcf_signal = (ACSR >> ACO) & 0x01;

         if (dcf77_config.current_dcf_signal == 1) {                         //comparator input below bandgap reference
           DCF_LED_PORT = DCF_LED_PORT & ~(1 << DCF_LED_ADDR);
         } else {
           DCF_LED_PORT = DCF_LED_PORT | (1 << DCF_LED_ADDR);
         }*/


        //try synchronization every DCF_UPDATE_CYCLE
        if (((real_time.current_minute % DCF77_UPDATE_CYCLE) == 0) && (real_time.current_second == 0)) {
          dcf77_config.dcf_state = dcf77_sync(dcf77_config.current_dcf_signal, DCF77_ACTIVE_LOW,
                                              &dcf77_time, DCF77_SAMPLE_TIME, DCF77_RESET_TIME);
        }


        //sync only if sync is not completed
        if (dcf77_config.dcf_state != DCF77_TIMESYNC_COMPLETED) {
          // dcf77_config.dcf_state = dcf77_sync(dcf77_config.current_dcf_signal, DCF77_ACTIVE_LOW,
          //                                     &dcf77_time, DCF77_SAMPLE_TIME, DCF77_GET_TIME);

          //seconds are synchronized
          if (dcf77_config.dcf_state == DCF77_STARTSYNC_COMPLETED) {

            //do NOT set the second value here to avoid
            //time errors by wrong detected starts
            //seconds will be written only when the complete
            //data (until year information) was received

            //preset the second value to the value after a completed cycle
            dcf77_time.current_second = DCF77_CYCLE_SECONDS;

          }

          //got minutes...
          if (dcf77_config.dcf_state == DCF77_MINUTE_COMPLETED) {
            if (dcf77_time.current_minute < 60) {       //check for plausibility

              //time data is always valid for the next minute
              //---> decrement minute and avoid updating if minute == 59
              if (dcf77_time.current_minute == 0) {
                dcf77_time.current_minute = 59;
                dcf77_config.dcf_state = dcf77_sync(dcf77_config.current_dcf_signal, DCF77_ACTIVE_LOW,
                                                    &dcf77_time, DCF77_SAMPLE_TIME, DCF77_RESET_TIME);
              } else {
                dcf77_time.current_minute--;
              }

              //write new data directly to DS1307
              rtc_data[0] = DS1307_I2C_ADRESS;       //slave address+Write-Bit
              rtc_data[1] = 0x01;
              rtc_data[2] = (dcf77_time.current_minute / 10) << 4;      //*10 minutes
              rtc_data[2] = rtc_data[2] | ((dcf77_time.current_minute - ((dcf77_time.current_minute / 10) * 10)) & 0x0f);

              //I2C communication
              i2c_retry_counter = I2C_MAX_RETRYS;
              return_value = I2C_IO_ERROR;
              while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
                return_value = i2c_master_data_io(rtc_data, 3, FALSE);
                i2c_retry_counter--;
                _delay_us(I2C_RETRY_WAIT_US);
              }

              //value not plausible --> restart time aquisition
            } else {
              dcf77_config.dcf_state = dcf77_sync(dcf77_config.current_dcf_signal, DCF77_ACTIVE_LOW,
                                                  &dcf77_time, DCF77_SAMPLE_TIME, DCF77_RESET_TIME);
            }
          }

          //got hours...
          if (dcf77_config.dcf_state == DCF77_HOUR_COMPLETED) {
            if (dcf77_time.current_hour < 24) {       //check for plausibility

              //write new data directly to DS1307
              rtc_data[0] = DS1307_I2C_ADRESS;       //slave address+Write-Bit
              rtc_data[1] = 0x02;
              rtc_data[2] = (dcf77_time.current_hour / 10) << 4;      //*10 minutes
              rtc_data[2] = rtc_data[2] | ((dcf77_time.current_hour - ((dcf77_time.current_hour / 10) * 10)) & 0x0f);

              //I2C communication
              i2c_retry_counter = I2C_MAX_RETRYS;
              return_value = I2C_IO_ERROR;
              while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
                return_value = i2c_master_data_io(rtc_data, 3, FALSE);
                i2c_retry_counter--;
                _delay_us(I2C_RETRY_WAIT_US);
              }

              //value not plausible --> restart time aquisition
            } else {
              dcf77_config.dcf_state = dcf77_sync(dcf77_config.current_dcf_signal, DCF77_ACTIVE_LOW,
                                                  &dcf77_time, DCF77_SAMPLE_TIME, DCF77_RESET_TIME);
            }
          }

          //got day
          if (dcf77_config.dcf_state == DCF77_DAY_COMPLETED) {
            if (dcf77_time.current_day < 32) {       //check for plausibility

              //write new data directly to DS1307
              rtc_data[0] = DS1307_I2C_ADRESS;       //slave address+Write-Bit
              rtc_data[1] = 0x04;
              rtc_data[2] = (dcf77_time.current_day / 10) << 4;      // *10 minutes
              rtc_data[2] = rtc_data[2] | ((dcf77_time.current_day - ((dcf77_time.current_day / 10) * 10)) & 0x0f);

              //I2C communication
              i2c_retry_counter = I2C_MAX_RETRYS;
              return_value = I2C_IO_ERROR;
              while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
                return_value = i2c_master_data_io(rtc_data, 3, FALSE);
                i2c_retry_counter--;
                _delay_us(I2C_RETRY_WAIT_US);
              }

              //value not plausible --> restart time aquisition
            } else {
              dcf77_config.dcf_state = dcf77_sync(dcf77_config.current_dcf_signal, DCF77_ACTIVE_LOW,
                                                  &dcf77_time, DCF77_SAMPLE_TIME, DCF77_RESET_TIME);
            }
          }

          //got weekday
          if (dcf77_config.dcf_state == DCF77_WEEKDAY_COMPLETED) {
            if ((dcf77_time.current_weekday > 0) && (dcf77_time.current_weekday < 8)) {   //check for plausibility

              //write new data directly to DS1307
              rtc_data[0] = DS1307_I2C_ADRESS;       //slave address+Write-Bit
              rtc_data[1] = 0x03;
              rtc_data[2] = dcf77_time.current_weekday & 0x0f;

              //I2C communication
              i2c_retry_counter = I2C_MAX_RETRYS;
              return_value = I2C_IO_ERROR;
              while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
                return_value = i2c_master_data_io(rtc_data, 3, FALSE);
                i2c_retry_counter--;
                _delay_us(I2C_RETRY_WAIT_US);
              }

              //value not plausible --> restart time aquisition
            } else {
              dcf77_config.dcf_state = dcf77_sync(dcf77_config.current_dcf_signal, DCF77_ACTIVE_LOW,
                                                  &dcf77_time, DCF77_SAMPLE_TIME, DCF77_RESET_TIME);
            }
          }

          //got month
          if (dcf77_config.dcf_state == DCF77_MONTH_COMPLETED) {
            if (dcf77_time.current_month < 13) {     //check for plausibility

              //write new data directly to DS1307
              rtc_data[0] = DS1307_I2C_ADRESS;       //slave address+Write-Bit
              rtc_data[1] = 0x05;
              rtc_data[2] = (dcf77_time.current_month / 10) << 4;      // *10 minutes
              rtc_data[2] = rtc_data[2] | ((dcf77_time.current_month - ((dcf77_time.current_month / 10) * 10)) & 0x0f);

              //I2C communication
              i2c_retry_counter = I2C_MAX_RETRYS;
              return_value = I2C_IO_ERROR;
              while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
                return_value = i2c_master_data_io(rtc_data, 3, FALSE);
                i2c_retry_counter--;
                _delay_us(I2C_RETRY_WAIT_US);
              }

              //value not plausible --> restart time aquisition
            } else {
              dcf77_config.dcf_state = dcf77_sync(dcf77_config.current_dcf_signal, DCF77_ACTIVE_LOW,
                                                  &dcf77_time, DCF77_SAMPLE_TIME, DCF77_RESET_TIME);
            }
          }

        }   //if (dcf77_config.dcf_state != DCF77_TIMESYNC_COMPLETED) {...



        // state change to DCF77_TIMESYNC_COMPLETED detected
        if ((prev_dcf77_state != DCF77_TIMESYNC_COMPLETED) && (dcf77_config.dcf_state == DCF77_TIMESYNC_COMPLETED)) {
          if (dcf77_time.current_year < 100) {    //check for plausibility

            //preset the second value to the value after a completed cycle
            dcf77_time.current_second = DCF77_CYCLE_SECONDS;

            //cycle completed - write the complete data again
            //to avoid errors, if a new minute has started
            rtc_data[0] = DS1307_I2C_ADRESS;            //slave address + Write-Bit
            rtc_data[1] = 0x00;
            rtc_data[2] = ((dcf77_time.current_second / 10) << 4) & 0x7f;      //*10 seconds
            rtc_data[2] = rtc_data[2] | ((dcf77_time.current_second - ((dcf77_time.current_second / 10) * 10)) & 0x0f);
            rtc_data[3] = (dcf77_time.current_minute / 10) << 4;      //*10 minutes
            rtc_data[3] = rtc_data[3] | ((dcf77_time.current_minute - ((dcf77_time.current_minute / 10) * 10)) & 0x0f);
            rtc_data[4] = (dcf77_time.current_hour / 10) << 4;      //*10 minutes
            rtc_data[4] = rtc_data[4] | ((dcf77_time.current_hour - ((dcf77_time.current_hour / 10) * 10)) & 0x0f);
            rtc_data[5] = dcf77_time.current_weekday & 0x0f;
            rtc_data[6] = (dcf77_time.current_day / 10) << 4;      // *10 minutes
            rtc_data[6] = rtc_data[6] | ((dcf77_time.current_day - ((dcf77_time.current_day / 10) * 10)) & 0x0f);
            rtc_data[7] = (dcf77_time.current_month / 10) << 4;      // *10 minutes
            rtc_data[7] = rtc_data[7] | ((dcf77_time.current_month - ((dcf77_time.current_month / 10) * 10)) & 0x0f);
            rtc_data[8] = (dcf77_time.current_year / 10) << 4;      // *10 minutes
            rtc_data[8] = rtc_data[8] | ((dcf77_time.current_year - ((dcf77_time.current_year / 10) * 10)) & 0x0f);

            //I2C communication
            i2c_retry_counter = I2C_MAX_RETRYS;
            return_value = I2C_IO_ERROR;
            while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
              return_value = i2c_master_data_io(rtc_data, 9, FALSE);
              i2c_retry_counter--;
              _delay_us(I2C_RETRY_WAIT_US);
            }

            //value not plausible --> restart time aquisition
          } else {
            dcf77_config.dcf_state = dcf77_sync(dcf77_config.current_dcf_signal, DCF77_ACTIVE_LOW,
                                                &dcf77_time, DCF77_SAMPLE_TIME, DCF77_RESET_TIME);
          }

#if RTC_LCD_DEBUG == 1
          lcd_set_cursor(7, 1);
          lcd_send_string("*", FALSE);
#endif
        }


        //save previous DCF state
        prev_dcf77_state = dcf77_config.dcf_state;



      }   //if (dcf77_config.dcf_enabled > 0) {...


      /***************************************************************************
      \internal
      Read RTC data from DS1307
      ***************************************************************************/

      if (time_scale == 25 || time_scale == 50 || time_scale == 75 || time_scale == 100 ||
          time_scale == 125 || time_scale == 150 || time_scale == 175 || time_scale == 200 ||
          time_scale == 225 || time_scale == 250) {

        //set address
        rtc_data[0] = DS1307_I2C_ADRESS;       //slave address+Write-Bit
        rtc_data[1] = 0x00;

        //I2C communication
        i2c_retry_counter = I2C_MAX_RETRYS;
        return_value = I2C_IO_ERROR;
        while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
          return_value = i2c_master_data_io(rtc_data, 2, FALSE);
          i2c_retry_counter--;
          _delay_us(I2C_RETRY_WAIT_US);
        }

        if (return_value == I2C_IO_SUCC) {

          rtc_data[0] = DS1307_I2C_ADRESS | 0x01;     //slave address+Read-Bit

          //I2C communication
          i2c_retry_counter = I2C_MAX_RETRYS;
          return_value = I2C_IO_ERROR;
          while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
            return_value = i2c_master_data_io(rtc_data, 8, FALSE);
            i2c_retry_counter--;
            _delay_us(I2C_RETRY_WAIT_US);
          }


          if (return_value == I2C_IO_SUCC) {

            //update current time data
            real_time.current_second = (10 * ((rtc_data[1] >> 4) & 0x07)) + (rtc_data[1] & 0x0f);
            real_time.current_minute = (10 * ((rtc_data[2] >> 4) & 0x07)) + (rtc_data[2] & 0x0f);
            if ((rtc_data[3] & 0x10) == 1) {   //AM/PM Mode
              real_time.current_hour = (10 * ((rtc_data[3] >> 4) & 0x01)) + (rtc_data[3] & 0x0f) +
                                       (12 * ((rtc_data[3] >> 5) & 0x01));
            } else {                           //24h Mode
              real_time.current_hour = (10 * ((rtc_data[3] >> 4) & 0x03)) + (rtc_data[3] & 0x0f);
            }
            real_time.current_weekday = rtc_data[4] & 0x07;
            real_time.current_day = (10 * ((rtc_data[5] >> 4) & 0x03)) + (rtc_data[5] & 0x0f);
            real_time.current_month = (10 * ((rtc_data[6] >> 4) & 0x01)) + (rtc_data[6] & 0x0f);
            real_time.current_year = (10 * ((rtc_data[7] >> 4) & 0x0f)) + (rtc_data[7] & 0x0f);



#if RTC_SAA1064_OUTPUT == 1

            //write time data to LED display
            //show date info after 10 minutes again
            if ((real_time.current_minute == 0) || (real_time.current_minute == 10) || (real_time.current_minute == 20) ||
                (real_time.current_minute == 30) || (real_time.current_minute == 40) || (real_time.current_minute == 50)) {
              if (real_time.current_second == 19) {
                rtc_show_date_saa1064(real_time.current_day, real_time.current_month, real_time.current_year, real_time.current_hour,
                                      real_time.current_minute, real_time.current_second, 1);
              } else if ((real_time.current_second >= 20) && (real_time.current_second < 32)) {
                rtc_show_date_saa1064(real_time.current_day, real_time.current_month, real_time.current_year, real_time.current_hour,
                                      real_time.current_minute, real_time.current_second, 0);
              } else if (real_time.current_second == 32) {
                rtc_show_time_saa1064(real_time.current_hour, real_time.current_minute, real_time.current_second, dcf77_config.dcf_state, 1);
              } else {
                rtc_show_time_saa1064(real_time.current_hour, real_time.current_minute, real_time.current_second, dcf77_config.dcf_state, 0);
              }
            } else {
              rtc_show_time_saa1064(real_time.current_hour, real_time.current_minute, real_time.current_second, dcf77_config.dcf_state, 0);
            }

#endif



#if RTC_LCD_TIMEDEBUG == 1
            //write data to LCD
            if (real_time.current_second != real_time.prev_second) {

              real_time.prev_second = real_time.current_second;

              //print time information to display
              lcd_cursor_home();
              //lcd_send_string("Zeit: ", FALSE);
              if (real_time.current_hour < 10) {
                lcd_send_string("0", FALSE);
              }
              itoa(real_time.current_hour, tmp_char, 10);
              lcd_send_string(tmp_char, FALSE);
              lcd_send_string(":", FALSE);
              if (real_time.current_minute < 10) {
                lcd_send_string("0", FALSE);
              }
              itoa(real_time.current_minute, tmp_char, 10);
              lcd_send_string(tmp_char, FALSE);
              lcd_send_string(":", FALSE);
              if (real_time.current_second < 10) {
                lcd_send_string("0", FALSE);
              }
              itoa(real_time.current_second, tmp_char, 10);
              lcd_send_string(tmp_char, FALSE);
              lcd_set_cursor(0, 1);
              if (real_time.current_day < 10) {
                lcd_send_string("0", FALSE);
              }
              itoa(real_time.current_day, tmp_char, 10);
              lcd_send_string(tmp_char, FALSE);
              if (dcf77_config.dcf_state == DCF77_TIMESYNC_COMPLETED) {
                lcd_send_string(".", FALSE);
              } else {
                if (ff_mem == 0) {
                  lcd_send_string(".", FALSE);
                } else {
                  lcd_send_string(" ", FALSE);
                }
              }
              if (real_time.current_month < 10) {
                lcd_send_string("0", FALSE);
              }
              itoa(real_time.current_month, tmp_char, 10);
              lcd_send_string(tmp_char, FALSE);
              if (dcf77_config.dcf_state == DCF77_TIMESYNC_COMPLETED) {
                lcd_send_string(".", FALSE);
              } else {
                if (ff_mem == 0) {
                  lcd_send_string(" ", FALSE);
                  ff_mem = 1;
                } else {
                  lcd_send_string(".", FALSE);
                  ff_mem = 0;
                }
              }
              if ((real_time.current_year + RTC_BASE_YEAR) < 10) {
                lcd_send_string("0", FALSE);
              }
              if (DISPLAY_NUMBER_COLUMNS > 9) {
                itoa(real_time.current_year + RTC_BASE_YEAR, tmp_char, 10);
              } else {
                itoa(real_time.current_year, tmp_char, 10);
              }
              lcd_send_string(tmp_char, FALSE);/*

              if (real_time.current_weekday == DCF77_MONDAY) {
                lcd_send_string(" Mo", FALSE);
              } else if (real_time.current_weekday == DCF77_TUESDAY) {
                lcd_send_string(" Di", FALSE);
              } else if (real_time.current_weekday == DCF77_WEDNESDAY) {
                lcd_send_string(" Mi", FALSE);
              } else if (real_time.current_weekday == DCF77_THURSDAY) {
                lcd_send_string(" Do", FALSE);
              } else if (real_time.current_weekday == DCF77_FRIDAY) {
                lcd_send_string(" Fr", FALSE);
              } else if (real_time.current_weekday == DCF77_SATURDAY) {
                lcd_send_string(" Sa", FALSE);
              } else if (real_time.current_weekday == DCF77_SUNDAY) {
                lcd_send_string(" So", FALSE);
              }*/

            }

#endif
          }

        }

      }     //if (time_scale == 25 || time_scale == 50 || time_scale == 75...


#if RTC_LCD_DEBUG == 1
      lcd_set_cursor(0, 3);
      lcd_send_string("   ", FALSE);
      lcd_set_cursor(0, 3);
      timer_value_start = TCNT0 - timer_value_start;
      itoa(timer_value_start, tmp_char, 10);
      lcd_send_string(tmp_char, FALSE);
      if ((time_scale != 0) && (prev_time_scale != MAX_TIME_SCALE)) {
        lcd_set_cursor(4, 3);
        lcd_send_string("   ", FALSE);
        lcd_set_cursor(4, 3);
        itoa(time_scale - prev_time_scale, tmp_char, 10);
        lcd_send_string(tmp_char, FALSE);
      }
      lcd_set_cursor(7, 3);
      itoa(dcf77_config.dcf_state, tmp_char, 10);
      lcd_send_string(tmp_char, FALSE);
#endif


      /***************************************************************************
      \internal
      Write values to control registers and switch on any I2C slave functionality
      ***************************************************************************/

      ctrl_data[1] = dcf77_config.current_dcf_signal & 0x01;
      ctrl_data[1] = ctrl_data[1] | ((dcf77_config.dcf_enabled & 0x01) << 1);
      ctrl_data[1] = ctrl_data[1] | ((dcf77_config.dcf_state & 0x0f) << 2);
      ctrl_data[2] = real_time.current_second & 0x3f;
      ctrl_data[2] = ctrl_data[2] | ((real_time.current_minute & 0x03) << 6);
      ctrl_data[3] = (real_time.current_minute >> 2) & 0x0f;
      ctrl_data[3] = ctrl_data[3] | ((real_time.current_month & 0x0f) << 4);
      ctrl_data[4] = real_time.current_hour & 0x1f;
      ctrl_data[4] = ctrl_data[4] | ((real_time.current_weekday & 0x07) << 5);
      ctrl_data[5] = real_time.current_day & 0x1f;
      ctrl_data[6] = real_time.current_year & 0x7f;


      //save timescale value
      prev_time_scale = time_scale;

    }  //if (prev_time_scale != time_scale) {...


  }     //while(1)


}


