/****************************************************************************/
/**
* \file saa1064.c
*
* \brief routines for LED control (via SAA1064)
*
* \author   Michael Skutek
* \date	    02/23/2013
* \version  0.2
*
* Platform:    AVR ATmega (tested on ATmega168)
*
* Include Files:
*       - avr/pgmspace.h:       for accessing program space
*       - util/delay.h:         time delays (part of avr-gcc)
*       - i2c.h                 I2C routines
*       - saa1064.h:            definitions and prototypes
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program. If not, see
* <http://www.gnu.org/licenses/>.
*
****************************************************************************/


/******************************************
	\internal
	Include files
*******************************************/


#include <avr/pgmspace.h>
#include <util/delay.h>

#include "basic_definitions.h"
#include "i2c.h"
#include "saa1064.h"

#if GLCD_ENABLED == 1
#include <stdlib.h>
#include "glcd.h"
#endif





/****************************************************************************/
/**
* Initializes a SAA1064 chip with a given mode, performs a short segment test,
* switches all segments to off and checks configuration (reads control
* byte and data bytes).\n
* This function is available for some startup tests and demo of the 7 segment
* LED displays. It is not really required for an initialization of the
* SAA1064 chip.
*
*
* \param [in] i2c_address      I2C adress of SAA1064 including write bit "0"
* \param [in] display_mode     SAA1064_STATIC_MODE or SAA1064_DYNAMIC_MODE
* \param [in] segment_current  value for segment current --> see datasheet:
*                              (Bits: X, X, X, X, X, C4, C5, C6)
*
* \return SAA1064_IO_SUCCESS:    communication was successful \n
*         SAA1064_IO_ERROR:      unknown or internal error occured \n
*         SAA1064_WRONG_ADDRESS: wrong I2C address --> see datasheet \n
*         SAA1064_INIT_ERROR:    no valid config data from SAA1064 received
*                                (initialization was NOT successful) \n
*         other values:          communication was NOT successful
*                                --> see "i2c.h"
*
* \date
* - Created 22.08.2010
* - Modified  23.02.2013
*
*
* \author
*   Michael Skutek, michael@skutek.de
*/
/****************************************************************************/

uint8_t saa1064_init(uint8_t i2c_address, uint8_t display_mode, uint8_t segment_current) {


  uint8_t return_value;
  uint8_t i2c_retry_counter;
  uint8_t i, k;
  static uint8_t saa1064_data[I2C_SAA1064_NUMBER_REG];   //contains slave address, instruction byte, control byte,
  //data digit 1, data digit 2, data digit 3, data digit 4

  //init value
  return_value = SAA1064_IO_ERROR;

  ///check given address values
  if ((i2c_address != I2C_SAA1064_ADDRESS0) &&
      (i2c_address != I2C_SAA1064_ADDRESS1) &&
      (i2c_address != I2C_SAA1064_ADDRESS2) &&
      (i2c_address != I2C_SAA1064_ADDRESS3)) {
    return SAA1064_WRONG_ADDRESS;
  }

  ///check given display mode
  if ((display_mode != SAA1064_STATIC_MODE) &&
      (display_mode != SAA1064_DYNAMIC_MODE)) {
    return SAA1064_WRONG_MODE;
  }

  ///check length of internal buffer
  if (I2C_SAA1064_NUMBER_REG < 7) {
    return SAA1064_IO_ERROR;
  }


  ///send configuration --> set control byte

  //if segment current if 0xff, the current value will be changed by an intervall
  if (segment_current == 0xff) {

    for (k = 0; k < 3; k++) {

      for (i = 0; i < 0x08; i++) {

        //prepare data buffer
        saa1064_data[0] = i2c_address;                              //I2C address + Write Bit
        saa1064_data[1] = 0x00;                                     //instruction byte (SAA1064 register address)
        saa1064_data[2] = 0x00 | ((i & 0x07) << 4);   //instruction byte (SAA1064 register address)
        saa1064_data[2] = saa1064_data[2] | 0x0e | display_mode;    //instruction byte (bits C0, C1, C2, C3, C4)
        //segment test, all digits are not blanked
        saa1064_data[3] = 0xff;                                     //digit 1
        saa1064_data[4] = 0xff;                                     //digit 2
        saa1064_data[5] = 0xff;                                     //digit 3
        saa1064_data[6] = 0xff;                                     //digit 4

        //I2C communication
        i2c_retry_counter = I2C_MAX_RETRYS;
        return_value = I2C_IO_ERROR;
        while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
          return_value = i2c_master_data_io(saa1064_data, 7, FALSE);
          i2c_retry_counter--;
          _delay_us(I2C_RETRY_WAIT_US);
        }

        //check for success
        if (return_value != I2C_IO_SUCC) {
          return return_value;                                       //error occured (bus blocked or no ACK etc.)
        }

        //wait for some time
        _delay_ms(50);

      }

      for (i = 0x07; i > 0; i--) {

        //prepare data buffer
        saa1064_data[0] = i2c_address;                              //I2C address + Write Bit
        saa1064_data[1] = 0x00;                                     //instruction byte (SAA1064 register address)
        saa1064_data[2] = 0x00 | ((i & 0x07) << 4);   //instruction byte (SAA1064 register address)
        saa1064_data[2] = saa1064_data[2] | 0x0e | display_mode;    //instruction byte (bits C0, C1, C2, C3, C4)
        //segment test, all digits are not blanked
        saa1064_data[3] = 0xff;                                     //digit 1
        saa1064_data[4] = 0xff;                                     //digit 2
        saa1064_data[5] = 0xff;                                     //digit 3
        saa1064_data[6] = 0xff;                                     //digit 4

        //I2C communication
        i2c_retry_counter = I2C_MAX_RETRYS;
        return_value = I2C_IO_ERROR;
        while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
          return_value = i2c_master_data_io(saa1064_data, 7, FALSE);
          i2c_retry_counter--;
          _delay_us(I2C_RETRY_WAIT_US);
        }

        //check for success
        if (return_value != I2C_IO_SUCC) {
          return return_value;                                       //error occured (bus blocked or no ACK etc.)
        }

        //wait for some time
        _delay_ms(50);

      }

    }

  } else {

    //prepare data buffer
    saa1064_data[0] = i2c_address;                              //I2C address + Write Bit
    saa1064_data[1] = 0x00;                                     //instruction byte (SAA1064 register address)
    saa1064_data[2] = 0x00 | ((segment_current & 0x07) << 4);   //instruction byte (SAA1064 register address)
    saa1064_data[2] = saa1064_data[2] | 0x0e | display_mode;    //instruction byte (bits C0, C1, C2, C3, C4)
    //segment test, all digits are not blanked
    saa1064_data[3] = 0xff;                                     //digit 1
    saa1064_data[4] = 0xff;                                     //digit 2
    saa1064_data[5] = 0xff;                                     //digit 3
    saa1064_data[6] = 0xff;                                     //digit 4

    //I2C communication
    i2c_retry_counter = I2C_MAX_RETRYS;
    return_value = I2C_IO_ERROR;
    while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
      return_value = i2c_master_data_io(saa1064_data, 7, FALSE);
      i2c_retry_counter--;
      _delay_us(I2C_RETRY_WAIT_US);
    }

    //check for success
    if (return_value != I2C_IO_SUCC) {
      return return_value;                                       //error occured (bus blocked or no ACK etc.)
    }

  }


  //wait for some time
  _delay_ms(SAA1064_INIT_DELAY_MS);


  ///send some init data --> set data bytes

  //prepare data buffer
  saa1064_data[0] = i2c_address;                              //I2C address + Write Bit
  saa1064_data[1] = 0x00;                                     //instruction byte (SAA1064 register address)
  saa1064_data[2] = 0x00 | ((segment_current & 0x07) << 4);   //instruction byte (SAA1064 register address)
  saa1064_data[2] = saa1064_data[2] | 0x06 | display_mode;    //instruction byte (bits C0, C1, C2, C3, C4)
  //NO segment test, all digits are not blanked
  saa1064_data[3] = pgm_read_byte(&numbers_bcd_table[1]);     //digit 1 --> display a "1"
  saa1064_data[4] = pgm_read_byte(&numbers_bcd_table[2]);     //digit 2 --> display a "2"
  saa1064_data[5] = pgm_read_byte(&numbers_bcd_table[3]);     //digit 3 --> display a "3"
  saa1064_data[6] = pgm_read_byte(&numbers_bcd_table[4]);     //digit 4 --> display a "4"

  //I2C communication
  i2c_retry_counter = I2C_MAX_RETRYS;
  return_value = I2C_IO_ERROR;
  while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
    return_value = i2c_master_data_io(saa1064_data, 7, FALSE);
    i2c_retry_counter--;
    _delay_us(I2C_RETRY_WAIT_US);
  }

  //check for success
  if (return_value != I2C_IO_SUCC) {
    return return_value;                                       //error occured (bus blocked or no ACK etc.)
  }

  //wait for some time
  _delay_ms(SAA1064_INIT_DELAY_MS);


  ///send some demo data --> set data bytes

  //prepare data buffer
  saa1064_data[0] = i2c_address;                              //I2C address + Write Bit
  saa1064_data[1] = 0x00;                                     //instruction byte (SAA1064 register address)
  saa1064_data[2] = 0x00 | ((segment_current & 0x07) << 4);   //instruction byte (SAA1064 register address)
  saa1064_data[2] = saa1064_data[2] | 0x06 | display_mode;    //instruction byte (bits C0, C1, C2, C3, C4)
  //NO segment test, all digits are not blanked

  for (k = 0; k < SAA1064_DEMO_LOOPS; k++) {
    for (i = 0; i < 6; i++) {

      saa1064_data[3] = 1 << i;     //digit 1 --> display only one segment
      saa1064_data[4] = 1 << i;     //digit 2 --> display only one segment
      saa1064_data[5] = 1 << i;     //digit 3 --> display only one segment
      saa1064_data[6] = 1 << i;     //digit 4 --> display only one segment

      //I2C communication
      i2c_retry_counter = I2C_MAX_RETRYS;
      return_value = I2C_IO_ERROR;
      while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
        return_value = i2c_master_data_io(saa1064_data, 7, FALSE);
        i2c_retry_counter--;
        _delay_us(I2C_RETRY_WAIT_US);
      }

      //check for success
      if (return_value != I2C_IO_SUCC) {
        return return_value;                                       //error occured (bus blocked or no ACK etc.)
      }

      //wait for some time
      _delay_ms(SAA1064_DEMO_DELAY_MS);

    }
  }


  ///send data --> clear all segments

  //prepare data buffer
  saa1064_data[0] = i2c_address;                              //I2C address + Write Bit
  saa1064_data[1] = 0x00;                                     //instruction byte (SAA1064 register address)
  saa1064_data[2] = 0x00 | ((segment_current & 0x07) << 4);   //instruction byte (SAA1064 register address)
  saa1064_data[2] = saa1064_data[2] | 0x06 | display_mode;    //instruction byte (bits C0, C1, C2, C3, C4)
  //NO segment test, all digits are not blanked
  saa1064_data[3] = 0x00;                                     //digit 1
  saa1064_data[4] = 0x00;                                     //digit 2
  saa1064_data[5] = 0x00;                                     //digit 3
  saa1064_data[6] = 0x00;                                     //digit 4

  //I2C communication
  i2c_retry_counter = I2C_MAX_RETRYS;
  return_value = I2C_IO_ERROR;
  while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
    return_value = i2c_master_data_io(saa1064_data, 7, FALSE);
    i2c_retry_counter--;
    _delay_us(I2C_RETRY_WAIT_US);
  }

  //check for success
  if (return_value != I2C_IO_SUCC) {
    return return_value;                                       //error occured (bus blocked or no ACK etc.)
  }


  return SAA1064_IO_SUCCESS;

}



/****************************************************************************/
/**
* Reads all data registers of the SAA1064 chip.\n
*
*
* \param [in] i2c_address      I2C adress of SAA1064 including read bit "1"
* \param [in] register_data    pointer to bytes which contains the status
*                              register data after reading
*                              (length: I2C_SAA1064_NUMBER_STAT)
*
* \return SAA1064_IO_SUCCESS:    communication was successful \n
*         SAA1064_IO_ERROR:      unknown or internal error occured \n
*         SAA1064_WRONG_ADDRESS: wrong I2C address --> see datasheet \n
*         other values:          communication was NOT successful
*                                --> see "i2c.h"
*
* \see I2C_SAA1064_NUMBER_STAT
*
* \date
* - Created 22.08.2010
* - Modified  23.02.2013
*
*
* \author
*   Michael Skutek, michael@skutek.de
*/
/****************************************************************************/

uint8_t saa1064_read_status(uint8_t i2c_address, uint8_t *register_data) {

  uint8_t return_value;
  uint8_t i2c_retry_counter;
  uint8_t i;
  static uint8_t saa1064_data[I2C_SAA1064_NUMBER_STAT];   //contains slave address and address byte

  //init value
  return_value = SAA1064_IO_ERROR;

  ///check given address values
  if ((i2c_address != I2C_SAA1064_ADDRESS0) &&
      (i2c_address != I2C_SAA1064_ADDRESS1) &&
      (i2c_address != I2C_SAA1064_ADDRESS2) &&
      (i2c_address != I2C_SAA1064_ADDRESS3)) {
    return SAA1064_WRONG_ADDRESS;
  }


  ///check length of internal buffer
  if (I2C_SAA1064_NUMBER_STAT < 2) {
    return SAA1064_IO_ERROR;
  }


  ///receive data --> set register address 0

  //prepare data buffer for setting read address
  saa1064_data[0] = i2c_address;                              //I2C address + Write Bit
  saa1064_data[1] = 0x00;                                     //instruction byte (SAA1064 register address)

  //I2C communication
  i2c_retry_counter = I2C_MAX_RETRYS;
  return_value = I2C_IO_ERROR;
  while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
    return_value = i2c_master_data_io(saa1064_data, 2, FALSE);
    i2c_retry_counter--;
    _delay_us(I2C_RETRY_WAIT_US);
  }

  //check for success
  if (return_value != I2C_IO_SUCC) {
    return return_value;                                       //error occured (bus blocked or no ACK etc.)
  }


  ///receive data --> set read bit and receive data

  //prepare data buffer
  saa1064_data[0] = i2c_address | 0x01;                       //I2C address + Read Bit

  //I2C communication
  i2c_retry_counter = I2C_MAX_RETRYS;
  return_value = I2C_IO_ERROR;
  while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
    return_value = i2c_master_data_io(saa1064_data, I2C_SAA1064_NUMBER_STAT, FALSE);
    i2c_retry_counter--;
    _delay_us(I2C_RETRY_WAIT_US);
  }

  //check for success
  if (return_value != I2C_IO_SUCC) {
    return return_value;                                       //error occured (bus blocked or no ACK etc.)
  }

  //copy received data into output buffer
  for (i = 0; i < I2C_SAA1064_NUMBER_STAT; i++) {
    register_data[i] = saa1064_data[i];
  }


  return SAA1064_IO_SUCCESS;

}



/****************************************************************************/
/**
* Sends digits (2 or 4 digits, depending on mode) to the SAA1064 chip.
*
*
* \param [in] i2c_address      I2C adress of SAA1064 including write bit "0"
* \param [in] display_mode     SAA1064_STATIC_MODE or SAA1064_DYNAMIC_MODE
* \param [in] segment_current  value for segment current --> see datasheet:
*                              (Bits: X, X, X, X, X, C4, C5, C6)
* \param [in] figure           value, which contains the figure to display
* \param [in] decimal_points   bitfield - enables decimal point for digit
* \param [in] clear            if > 0, it clears the display
*
* \return SAA1064_IO_SUCCESS:    communication was successful \n
*         SAA1064_IO_ERROR:      unknown or internal error occured \n
*         SAA1064_WRONG_ADDRESS: wrong I2C address --> see datasheet \n
*         other values:          communication was NOT successful
*                                --> see "i2c.h"
*
* \see value definition of numbers_bcd_table in file saa1064.h
*
* \date
* - Created 22.08.2010
* - Modified  23.02.2013
*
*
* \author
*   Michael Skutek, michael@skutek.de
*/
/****************************************************************************/

uint8_t saa1064_send_digits(uint8_t i2c_address, uint8_t display_mode, uint8_t segment_current,
                            uint16_t figure, uint8_t decimal_points, uint8_t clear) {


  uint8_t return_value;
  uint8_t i2c_retry_counter;
  uint16_t digit;
  static uint8_t saa1064_data[I2C_SAA1064_NUMBER_REG];   //contains slave address and address byte

  //init value
  return_value = SAA1064_IO_ERROR;

  ///check given address values
  if ((i2c_address != I2C_SAA1064_ADDRESS0) &&
      (i2c_address != I2C_SAA1064_ADDRESS1) &&
      (i2c_address != I2C_SAA1064_ADDRESS2) &&
      (i2c_address != I2C_SAA1064_ADDRESS3)) {
    return SAA1064_WRONG_ADDRESS;
  }


  ///check length of internal buffer
  if (I2C_SAA1064_NUMBER_REG < 7) {
    return SAA1064_IO_ERROR;
  }

  ///send some init data --> set data bytes

  //prepare data buffer
  saa1064_data[0] = i2c_address;                              //I2C address + Write Bit
  saa1064_data[1] = 0x00;                                     //instruction byte (SAA1064 register address)
  saa1064_data[2] = 0x00 | ((segment_current & 0x07) << 4);   //instruction byte (SAA1064 register address)
  saa1064_data[2] = saa1064_data[2] | 0x06 | display_mode;    //instruction byte (bits C0, C1, C2, C3, C4)
  //NO segment test, all digits are not blanked

  //fill data depending on mode
  if (clear > 0) {

    if (display_mode == SAA1064_STATIC_MODE) {

      digit = figure % 10;
      if ((decimal_points & 0x01) > 0) {            //decimal point for digit requested?
        saa1064_data[3] = pgm_read_byte(&numbers_bcd_table[BCD_TABLE_DECIMAL_POINT_VALUE]);  //digit 1
      } else {
        saa1064_data[3] = pgm_read_byte(&numbers_bcd_table[BCD_TABLE_OFF_VALUE]);  //digit 1
      }
      digit = figure % 100 / 10;
      if ((decimal_points & 0x02) > 0) {            //decimal point for digit requested?
        saa1064_data[4] = pgm_read_byte(&numbers_bcd_table[BCD_TABLE_DECIMAL_POINT_VALUE]);  //digit 2
      } else {
        saa1064_data[4] = pgm_read_byte(&numbers_bcd_table[BCD_TABLE_OFF_VALUE]);  //digit 2
      }
      saa1064_data[5] = pgm_read_byte(&numbers_bcd_table[BCD_TABLE_ERROR_VALUE]);     //digit 3 --> display a "-"
      saa1064_data[6] = pgm_read_byte(&numbers_bcd_table[BCD_TABLE_ERROR_VALUE]);     //digit 4 --> display a "-"

    } else {

      digit = figure % 10;
      if ((decimal_points & 0x01) > 0) {            //decimal point for digit requested?
        saa1064_data[3] = pgm_read_byte(&numbers_bcd_table[BCD_TABLE_DECIMAL_POINT_VALUE]);  //digit 1
      } else {
        saa1064_data[3] = pgm_read_byte(&numbers_bcd_table[BCD_TABLE_OFF_VALUE]);  //digit 1
      }
      digit = figure % 100 / 10;
      if ((decimal_points & 0x02) > 0) {            //decimal point for digit requested?
        saa1064_data[4] = pgm_read_byte(&numbers_bcd_table[BCD_TABLE_DECIMAL_POINT_VALUE]);  //digit 2
      } else {
        saa1064_data[4] = pgm_read_byte(&numbers_bcd_table[BCD_TABLE_OFF_VALUE]);  //digit 2
      }
      digit = figure % 1000 / 100;
      if ((decimal_points & 0x04) > 0) {            //decimal point for digit requested?
        saa1064_data[5] = pgm_read_byte(&numbers_bcd_table[BCD_TABLE_DECIMAL_POINT_VALUE]);  //digit 4
      } else {
        saa1064_data[5] = pgm_read_byte(&numbers_bcd_table[BCD_TABLE_OFF_VALUE]);  //digit 4
      }
      digit = figure % 10000 / 1000;
      if ((decimal_points & 0x08) > 0) {            //decimal point for digit requested?
        saa1064_data[6] = pgm_read_byte(&numbers_bcd_table[BCD_TABLE_DECIMAL_POINT_VALUE]);  //digit 5
      } else {
        saa1064_data[6] = pgm_read_byte(&numbers_bcd_table[BCD_TABLE_OFF_VALUE]);  //digit 5
      }

    }

  } else if (display_mode == SAA1064_STATIC_MODE) {

    digit = figure % 10;
    if ((decimal_points & 0x01) > 0) {            //decimal point for digit requested?
      saa1064_data[3] = pgm_read_byte(&numbers_bcd_table[digit + BCD_DECIMAL_POINT_OFFSET]);  //digit 1
    } else {
      saa1064_data[3] = pgm_read_byte(&numbers_bcd_table[digit]);  //digit 1
    }
    digit = figure % 100 / 10;
    if ((decimal_points & 0x02) > 0) {            //decimal point for digit requested?
      saa1064_data[4] = pgm_read_byte(&numbers_bcd_table[digit + BCD_DECIMAL_POINT_OFFSET]);  //digit 2
    } else {
      saa1064_data[4] = pgm_read_byte(&numbers_bcd_table[digit]);  //digit 2
    }
    saa1064_data[5] = pgm_read_byte(&numbers_bcd_table[BCD_TABLE_ERROR_VALUE]);     //digit 3 --> display a "-"
    saa1064_data[6] = pgm_read_byte(&numbers_bcd_table[BCD_TABLE_ERROR_VALUE]);     //digit 4 --> display a "-"

  } else {

    digit = figure % 10;
    if ((decimal_points & 0x01) > 0) {            //decimal point for digit requested?
      saa1064_data[3] = pgm_read_byte(&numbers_bcd_table[digit + BCD_DECIMAL_POINT_OFFSET]);  //digit 1
    } else {
      saa1064_data[3] = pgm_read_byte(&numbers_bcd_table[digit]);  //digit 1
    }
    digit = figure % 100 / 10;
    if ((decimal_points & 0x02) > 0) {            //decimal point for digit requested?
      saa1064_data[4] = pgm_read_byte(&numbers_bcd_table[digit + BCD_DECIMAL_POINT_OFFSET]);  //digit 2
    } else {
      saa1064_data[4] = pgm_read_byte(&numbers_bcd_table[digit]);  //digit 2
    }
    digit = figure % 1000 / 100;
    if ((decimal_points & 0x04) > 0) {            //decimal point for digit requested?
      saa1064_data[5] = pgm_read_byte(&numbers_bcd_table[digit + BCD_DECIMAL_POINT_OFFSET]);  //digit 4
    } else {
      saa1064_data[5] = pgm_read_byte(&numbers_bcd_table[digit]);  //digit 4
    }
    digit = figure % 10000 / 1000;
    if ((decimal_points & 0x08) > 0) {            //decimal point for digit requested?
      saa1064_data[6] = pgm_read_byte(&numbers_bcd_table[digit + BCD_DECIMAL_POINT_OFFSET]);  //digit 5
    } else {
      saa1064_data[6] = pgm_read_byte(&numbers_bcd_table[digit]);  //digit 5
    }

  }

  //I2C communication
  i2c_retry_counter = I2C_MAX_RETRYS;
  return_value = I2C_IO_ERROR;
  while (return_value != I2C_IO_SUCC && i2c_retry_counter > 0) {
    return_value = i2c_master_data_io(saa1064_data, 7, FALSE);
    i2c_retry_counter--;
    _delay_us(I2C_RETRY_WAIT_US);
  }

  //check for success
  if (return_value != I2C_IO_SUCC) {
    return return_value;                                       //error occured (bus blocked or no ACK etc.)
  }

  return SAA1064_IO_SUCCESS;

}
